@@ -2,27 +2,37 @@
 /*                                                                          */
 /*                         GNAT COMPILER COMPONENTS                         */
 /*                                                                          */
-/*                               P T Y P E S                                */
+/*                            A - G E T T T Y                               */
 /*                                                                          */
 /*                                  Body                                    */
 /*                                                                          */
-/*                             $Revision: 1.3 $                             */
+/*                             $Revision: 1.5 $                             */
 /*                                                                          */
-/*             Copyright (c) 1992,1993, NYU, All Rights Reserved            */
+/*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
-/* terms  of the GNU  General  Public  License  as  published  by the  Free */
-/* Software  Foundation;  either version 2,  or (at your option)  any later */
-/* version.  GNAT is distributed  in the hope  that it will be useful,  but */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT- */
-/* ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public */
-/* License  for  more details.  You should have received  a copy of the GNU */
-/* General Public License along with GNAT;  see file COPYING. If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* terms of the  GNU General Public License as published  by the Free Soft- */
+/* ware  Foundation;  either version 2,  or (at your option) any later ver- */
+/* sion.  GNAT is distributed in the hope that it will be useful, but WITH- */
+/* OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY */
+/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
+/* for  more details.  You should have  received  a copy of the GNU General */
+/* Public License  distributed with GNAT;  see file COPYING.  If not, write */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* As a  special  exception,  if you  link  this file  with other  files to */
+/* produce an executable,  this file does not by itself cause the resulting */
+/* executable to be covered by the GNU General Public License. This except- */
+/* ion does not  however invalidate  any other reasons  why the  executable */
+/* file might be covered by the  GNU Public License.                        */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
-/* Functions for retrieving target types. */
+/* Functions for retrieving target types. See Ada package Get_Targ */
 
 #include "config.h"
 #include "a-ada.h"
@@ -144,16 +154,11 @@ get_target_maximum_alignment ()
   return BIGGEST_ALIGNMENT / BITS_PER_UNIT;
 }
 
-/* It doesn't matter what value this function returns, so long as it
-   returns a different value for each endianness possibility.  */
-
 #ifndef FLOAT_WORDS_BIG_ENDIAN
 #define FLOAT_WORDS_BIG_ENDIAN WORDS_BIG_ENDIAN
 #endif
 
-Nat
-get_target_endianness ()
-{
-  return (8 * FLOAT_WORDS_BIG_ENDIAN + 4 * WORDS_BIG_ENDIAN
-	  + 2 * BYTES_BIG_ENDIAN + BITS_BIG_ENDIAN);
-}
+Nat get_float_words_be () {return FLOAT_WORDS_BIG_ENDIAN;}
+Nat get_words_be       () {return WORDS_BIG_ENDIAN;}
+Nat get_bytes_be       () {return BYTES_BIG_ENDIAN;}
+Nat get_bits_be        () {return BITS_BIG_ENDIAN;}
