@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                             $Revision: 1.67 $                            */
+/*                             $Revision: 1.94 $                            */
 /*                                                                          */
-/*           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          */
+/*   Copyright (C) 1992,1993,1994,1995,1996 Free Software Foundation, Inc.  */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -18,7 +18,17 @@
 /* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
 /* for  more details.  You should have  received  a copy of the GNU General */
 /* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* As a  special  exception,  if you  link  this file  with other  files to */
+/* produce an executable,  this file does not by itself cause the resulting */
+/* executable to be covered by the GNU General Public License. This except- */
+/* ion does not  however invalidate  any other reasons  why the  executable */
+/* file might be covered by the  GNU Public License.                        */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
@@ -46,6 +56,14 @@
 #include "a-rtree.h"
 #include "flags.h"
 
+/* This is needed for Linux since "setjmp" is a macro defined to be a function
+   of a different name.  This file MUST NOT be included for other systems
+   because of the potential of conflicts with the way we kludge uses of
+   setjmp at the moment.  */
+#ifdef linux
+#include <setjmp.h>
+#endif
+
 extern char *xmalloc ();
 extern char *main_input_filename;
 
@@ -90,9 +108,6 @@ char *gnat_tree_code_name[] = {
 extern int gnat_argc;
 extern char **gnat_argv;
 
-/* Root node of the tree read in.  Used only by yyparse.  */
-Node_Id gnat_root;
-
 /* Global Variables Expected by gcc: */
 
 char *language_string = "GNU Ada";
@@ -114,7 +129,11 @@ yyparse ()
   ada__bind ();
 
   /* Set up to catch unhandled exceptions.  */
+#if defined (sparc) && defined (__svr4__)
+  if (sigsetjmp (jmpbuf))
+#else
   if (setjmp (jmpbuf))
+#endif
     abort ();
 
   system__task_specific_data__set_jmpbuf_address (jmpbuf);
@@ -152,18 +171,28 @@ int
 lang_decode_option (p)
      char *p;
 {
+  if (!gnat_argc) init_gnat_args ();
 #if defined (WINNT) || defined (__EMX__)
   if (strnicmp (p, "-gnat", 5))
 #else
   if (strncasecmp (p, "-gnat", 5))
 #endif
-    /* we assume for the moment that all front end options are passed
-       with "gnat" prefix */
-    return 0;
+    if (!strncmp (p, "-I", 2))
+       {
+       /* pass the -I switches as-is */
+       gnat_argv[gnat_argc] = p;
+       gnat_argc ++;
+       return 1;
+       }
   else
     {
-      if (!gnat_argc) init_gnat_args ();
+       /* we assume for the moment that all other front end options
+          are passed with "gnat" prefix */
+       return 0;
+       }
 
+  else
+    {
       /* recopy the switches without the 'gnat' prefix */
 
       gnat_argv[gnat_argc] =  (char *) malloc (strlen (p) - 3);
@@ -310,45 +339,76 @@ gnat_expand_expr (exp, target, tmode, modifier)
 
     case UNCHECKED_CONVERT_EXPR:
       /* If the input and output are both the same mode (usually BLKmode),
-	 just return the expanded input since we want just the bits.  */
-      if (TYPE_MODE (type) == TYPE_MODE (TREE_TYPE (TREE_OPERAND (exp, 0))))
+	 just return the expanded input since we want just the bits.  But
+	 we can't do this if the output is more strictly aligned than
+	 the input.  */
+      if (TYPE_MODE (type) == TYPE_MODE (TREE_TYPE (TREE_OPERAND (exp, 0)))
+	  && (TYPE_ALIGN (type)
+	      <= TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0)))))
 	new = TREE_OPERAND (exp, 0);
 
       /* If either mode is BLKmode, memory will be involved, so do this
-	 via pointer punning.  */
-      else if (TYPE_MODE (type) == BLKmode
-	       || TYPE_MODE (TREE_TYPE (TREE_OPERAND (exp, 0))) == BLKmode)
-	new
-	  = build_unary_op (INDIRECT_REF, NULL_TREE,
-			    convert (build_pointer_type (type),
-				     build_unary_op (ADDR_EXPR, NULL_TREE,
-						     TREE_OPERAND (exp, 0))));
+	 via pointer punning.  Likewise, this doesn't work if there
+	 is an alignment issue.  But we must do it for types that are known
+	 to be aligned properly.  */
+      else if ((TYPE_MODE (type) == BLKmode
+		|| TYPE_MODE (TREE_TYPE (TREE_OPERAND (exp, 0))) == BLKmode)
+	       && ((TYPE_ALIGN (type)
+		    <= TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0))))
+		   || TYPE_ALIGN_OK_P (type)
+		   || TYPE_ALIGN_OK_P (TREE_TYPE (TREE_OPERAND (exp, 0)))))
+	{
+	  new = build_unary_op (INDIRECT_REF, NULL_TREE,
+				convert
+				(build_pointer_type (type),
+				 build_unary_op (ADDR_EXPR, NULL_TREE,
+						 TREE_OPERAND (exp, 0))));
+	  result = expand_expr (new, target, tmode, modifier);
+	  if (GET_CODE (result) != MEM)
+	    gigi_abort (204);
+
+	  /* Since this is really the underlying object, set the flags from
+	     the underlying type.  */
+	  MEM_VOLATILE_P (result) = TREE_THIS_VOLATILE (TREE_OPERAND (exp, 0));
+	  RTX_UNCHANGING_P (result) = TREE_READONLY (TREE_OPERAND (exp, 0));
+	  MEM_IN_STRUCT_P (result)
+	    = AGGREGATE_TYPE_P (TREE_TYPE (TREE_OPERAND (exp, 0)));
+	  return result;
+	}
 
       /* Otherwise make a union of the two types, convert to the union, and
 	 extract the other value.  */
       else
 	{
-	  /* Note that copy_node puts objects in current_obstack and we
-	     take advantage of that here since we want these objects to
-	     all be in the momentary obstack.  */
-	  tree in_type = TREE_TYPE (TREE_OPERAND (exp, 0));
-	  tree union_type = copy_node (unchecked_union_node);
-	  tree in_field
-	    = create_field_decl ("in", in_type, union_type, 0, -1, 0);
-	  tree out_field
-	    = create_field_decl ("out", type, union_type, 0, -1, 0);
+	  tree in_type, union_type, in_field, out_field;
 
-	  TYPE_MAIN_VARIANT (union_type) = union_type;
-	  finish_record_type (union_type,
-			      chainon (chainon (NULL_TREE, in_field),
-				       out_field),
-			      0);
+	  /* If this is inside the LHS of an assignment, this would generate
+	     bad code, so abort.  */
+	  if (TREE_ADDRESSABLE (exp))
+	    gigi_abort (202);
+
+	  in_type = TREE_TYPE (TREE_OPERAND (exp, 0));
+	  union_type = make_node (UNION_TYPE);
+	  in_field = create_field_decl ("in", in_type, union_type, 0, -1, 0);
+	  out_field = create_field_decl ("out", type, union_type, 0, -1, 0);
+
+	  TYPE_FIELDS (union_type) = chainon (in_field, out_field);
+	  layout_type (union_type);
+
+	  /* Though this is a "union", we can treat its size as that of
+	     the output type in case the size of the input type is variable.
+	     If the output size is a variable, use the input size.  */
+	  TYPE_SIZE (union_type) = TYPE_SIZE (type);
+	  if (TREE_CODE (TYPE_SIZE (type)) != INTEGER_CST
+	      && TREE_CODE (TYPE_SIZE (in_type)) == INTEGER_CST)
+	    TYPE_SIZE (union_type) = TYPE_SIZE (in_type);
 
 	  new = build (COMPONENT_REF, type,
 		       build1 (CONVERT_EXPR, union_type,
 			       TREE_OPERAND (exp, 0)),
 		       out_field);
 	}
+
       break;
 
     case NULL_EXPR:
@@ -361,8 +421,10 @@ gnat_expand_expr (exp, target, tmode, modifier)
 	  int size = int_size_in_bytes (type);
 	  rtx tem;
 
+	  /* We really can't handle variable-sized objects here, but we're
+	     not going to do anything with it, so just allocate 1 byte.  */
 	  if (size == -1)
-	    gigi_abort (202);
+	    size = 1;
 
 	  tem = assign_stack_temp (TYPE_MODE (type), size, 0);
 	  MEM_IN_STRUCT_P (tem) = AGGREGATE_TYPE_P (type);
@@ -422,6 +484,46 @@ make_transform_expr (gnat_node, gnu_type)
   TREE_COMPLEXITY (gnu_result) = gnat_node;
   return gnu_result;
 }
+
+/* Record the current code position in GNAT_NODE.  */
+
+void
+record_code_position (gnat_node)
+     Node_Id gnat_node;
+{
+  if (global_bindings_p ())
+    save_gnu_tree (gnat_node, get_elaboration_location (), 1);
+  else
+    /* Always emit another insn in case marking the last insn
+       addressable needs some fixups.  */
+    save_gnu_tree (gnat_node,
+		   (tree) emit_note (NULL_PTR, NOTE_INSN_DELETED), 1);
+}
+
+/* Insert the code for GNAT_NODE at the position saved for that node.  */
+
+void
+insert_code_for (gnat_node)
+     Node_Id gnat_node;
+{
+  if (global_bindings_p ())
+    {
+      push_pending_elaborations ();
+      insert_elaboration_list (get_gnu_tree (gnat_node));
+      pop_pending_elaborations ();
+      gnat_to_code (gnat_node);
+    }
+  else
+    {
+      rtx insns;
+
+      start_sequence ();
+      gnat_to_code (gnat_node);
+      insns = get_insns ();
+      end_sequence ();
+      emit_insns_after (insns, (rtx) get_gnu_tree (gnat_node));
+    }
+}
 
 /* Performs whatever initialization steps needed by the language-dependent
    lexical analyzer.
@@ -498,7 +600,9 @@ int
 pass_by_ref (gnu_type)
      tree gnu_type;
 {
-  /* We pass only BLKmode and unconstrained objects by reference.  */
+  /* We pass only BLKmode, unconstrained objects, and those required
+     by the language by reference.  */
   return (TREE_CODE (gnu_type) == UNCONSTRAINED_ARRAY_TYPE
-	  || TYPE_MODE (gnu_type) == BLKmode);
+	  || TYPE_MODE (gnu_type) == BLKmode
+	  || (AGGREGATE_TYPE_P (gnu_type) && TYPE_BY_REFERENCE_P (gnu_type)));
 }
