@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.21 $                             */
+/*                            $Revision: 1.23 $                             */
 /*                                                                          */
-/*           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          */
+/*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -18,7 +18,11 @@
 /* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
 /* for  more details.  You should have  received  a copy of the GNU General */
 /* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
@@ -82,7 +86,8 @@ Get_Decoded_Name_String (Id)
   return Name_Buffer;
 }
 
-/* Likewise, but also upper-case the string.  */
+/* Likewise, but also upper-case the string unless it is a character
+   literal.  This is used for building the enumeration literal table */
 
 extern void casing__set_all_upper_case PROTO ((void));
 
@@ -91,7 +96,8 @@ Get_Upper_Decoded_Name_String (Id)
      Name_Id Id;
 {
   namet__get_decoded_name_string (Id);
-  casing__set_all_upper_case ();
+  if (Name_Buffer [0] != '\'')
+    casing__set_all_upper_case ();
   Name_Buffer [Name_Len] = 0;
   return Name_Buffer;
 }
