@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                            $Revision: 1.110 $                             */
+/*                            $Revision: 1.148 $                            */
 /*                                                                          */
-/*           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          */
+/*   Copyright (C) 1992,1993,1994,1995,1996 Free Software Foundation, Inc.  */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -18,7 +18,11 @@
 /* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
 /* for  more details.  You should have  received  a copy of the GNU General */
 /* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
@@ -71,11 +75,12 @@ tree jmpbuf_ptr_type;
 tree get_jmpbuf_decl;
 tree set_jmpbuf_decl;
 tree get_excptr_decl;
+tree set_except_occ_decl;
 tree raise_decl;
 tree raise_nodefer_decl;
 tree setjmp_decl;
 tree raise_constraint_error_decl;
-tree unchecked_union_node;
+tree raise_program_error_decl;
 
 tree integer_zero_node;
 tree integer_one_node;
@@ -83,7 +88,9 @@ tree null_pointer_node;
 
 tree current_function_decl = NULL;
 
-static int contains_placeholder_except_p PROTO((tree, tree));
+static int contains_placeholder_except_p	PROTO((tree, tree));
+static tree convert_to_fat_pointer		PROTO((tree, tree));
+static tree convert_to_thin_pointer		PROTO((tree, tree));
 
 /* Routines to Associate and Retrieve GCC Nodes with Gnat Nodes: */
 
@@ -92,6 +99,19 @@ static int contains_placeholder_except_p PROTO((tree, tree));
    of `save_gnu_tree' for more info.  */
 static tree *associate_gnat_to_gnu;
 
+/* This listhead is used to record any global objects that need elaboration.
+   TREE_PURPOSE is the variable to be elaborated and TREE_VALUE is the
+   initial value to assign.  */
+
+static tree pending_elaborations;
+
+/* This stack allows us to momentarily switch to generating elaboration
+   lists for an inner context.  */
+
+static struct e_stack {struct e_stack *next; tree elab_list; } *elist_stack;
+
+extern struct obstack *saveable_obstack;
+
 /* Initialize the association of GNAT nodes to GCC trees.  */
 
 void
@@ -105,6 +125,8 @@ init_gnat_to_gnu ()
     associate_gnat_to_gnu [gnat_node]   = NULL_TREE;
 
   associate_gnat_to_gnu   -= First_Node_Id;
+
+  pending_elaborations = build_tree_list (NULL_TREE, NULL_TREE);
 }
 
 /* GNAT_ENTITY is a GNAT tree node for an entity.   GNU_DECL is the GCC tree
@@ -405,9 +427,14 @@ tree
 pushdecl (decl)
      tree decl;
 {
+  struct binding_level *b = current_binding_level;
+
   /* External objects aren't nested, other objects may be.  */
   if (DECL_EXTERNAL (decl))
-    DECL_CONTEXT (decl) = 0;
+    {
+      DECL_CONTEXT (decl) = 0;
+      b = global_binding_level;
+    }
   else
     DECL_CONTEXT (decl) = current_function_decl;
 
@@ -420,22 +447,25 @@ pushdecl (decl)
   if (TREE_CODE (decl) != TYPE_DECL
       || TREE_CODE (TREE_TYPE (decl)) != UNCONSTRAINED_ARRAY_TYPE)
     {
-      TREE_CHAIN (decl) = current_binding_level->names;
-      current_binding_level->names = decl;
+      TREE_CHAIN (decl) = b->names;
+      b->names = decl;
     }
 
-  /* For the declartion of a type, set its name if it either is not already
+  /* For the declaration of a type, set its name if it either is not already
      set or is an implicit type name.  We'd rather have the type named with a
      real name and all the pointer types to the same object have the same
      POINTER_TYPE node.  Code in this function in c-decl.c makes a copy
      of the type node here, but that may cause us trouble with incomplete
-     types, so let's not try it (at least for now).  */
+     types, so let's not try it (at least for now).  Ensure we don't set a
+     name if TYPE is not in the same obstack as DECL would have been placed. */
 
   if (TREE_CODE (decl) == TYPE_DECL
       && DECL_NAME (decl) != 0
       && (TYPE_NAME (TREE_TYPE (decl)) == 0
 	  || (TREE_CODE (TYPE_NAME (TREE_TYPE (decl))) == IDENTIFIER_NODE
-	      && *(IDENTIFIER_POINTER (TYPE_NAME (TREE_TYPE (decl)))) == 'T')))
+	      && *(IDENTIFIER_POINTER (TYPE_NAME (TREE_TYPE (decl)))) == 'T'))
+      && (TREE_PERMANENT (decl)
+	  || saveable_obstack == TYPE_OBSTACK (TREE_TYPE (decl))))
     TYPE_NAME (TREE_TYPE (decl)) = decl;
 
   return decl;
@@ -539,6 +569,17 @@ init_decl_processing ()
 							   endlink)),
 			   NULL_TREE, 0, 1, 1, 0, NULL_TREE);
 
+/* set_except_occ is a function that initialize the current exception 
+   occurrence of a handler with the message saved in the task-specific data */
+
+  set_except_occ_decl
+    = create_subprog_decl ("__set_except_occ", NULL_PTR,
+			   build_function_type (void_type_node,
+						tree_cons (NULL_TREE,
+							   ptr_void_type_node,
+							   endlink)),
+			   NULL_TREE, 0, 1, 1, 0, NULL_TREE);
+
   /* Make the types and functions used for exception processing.  
      We assume here that a jmp_buf is no more than 100 size_t entries.  */
   jmpbuf_type
@@ -586,7 +627,7 @@ init_decl_processing ()
 
   raise_nodefer_decl
     = create_subprog_decl
-      ("__gnat_raise_nodefer", NULL_PTR,
+      ("__gnat_raise_nodefer_with_msg", NULL_PTR,
        build_function_type (void_type_node,
 			    tree_cons (NULL_TREE,
 				       build_pointer_type (except_type_node),
@@ -601,16 +642,24 @@ init_decl_processing ()
      a jmpbuf.  */
   setjmp_decl
     = create_subprog_decl
+#ifdef linux
+      ("__setjmp", NULL_PTR,
+#else
 #ifdef WINNT
       ("_setjmp", NULL_PTR,
+#else
+#if defined (sparc) && defined (__svr4__)
+      ("sigsetjmp", NULL_PTR,
 #else
       ("setjmp", NULL_PTR,
+#endif
+#endif
 #endif
        build_function_type (integer_type_node,
 			    tree_cons (NULL_TREE,  jmpbuf_ptr_type, endlink)),
        NULL_TREE, 0, 1, 1, 0, NULL_TREE);
 
-  /* __gnat_raise_constraint takes no operands and never returns.  */
+  /* __gnat_raise_constraint_error takes no operands and never returns.  */
   raise_constraint_error_decl
     = create_subprog_decl
       ("__gnat_raise_constraint_error", NULL_PTR,
@@ -620,9 +669,15 @@ init_decl_processing ()
   TREE_THIS_VOLATILE (raise_constraint_error_decl) = 1;
   TREE_SIDE_EFFECTS (raise_constraint_error_decl) = 1;
 
-  /* Make a UNION_TYPE that will be copied to form a union to be used to
-     do an unchecked conversion.  */
-  unchecked_union_node = make_node (UNION_TYPE);
+  /* Likewise for __gnat_raise_program_error.  */
+  raise_program_error_decl
+    = create_subprog_decl
+      ("__gnat_raise_program_error", NULL_PTR,
+       build_function_type (void_type_node, endlink),
+       NULL_TREE, 0, 1, 1, 0, NULL_TREE);
+
+  TREE_THIS_VOLATILE (raise_program_error_decl) = 1;
+  TREE_SIDE_EFFECTS (raise_program_error_decl) = 1;
 }
 
 /* This routine is called in tree.c to print an error message for invalid use
@@ -651,13 +706,16 @@ finish_incomplete_decl (dont_care)
 /* Given a record type (RECORD_TYPE) and a chain of FIELD_DECL
    nodes (FIELDLIST), finish constructing the record or union type. 
    If HAS_REP is nonzero, this record has a rep clause; don't call
-   layout_type but merely set the size and alignment ourselves.  */
+   layout_type but merely set the size and alignment ourselves. 
+   If DEFER_DEBUG is nonzero, do not call the debugging routines
+   on this type; it will be done later. */
 
 void
-finish_record_type (record_type, fieldlist, has_rep)
+finish_record_type (record_type, fieldlist, has_rep, defer_debug)
      tree record_type;
      tree fieldlist;
      int has_rep;
+     int defer_debug;
 {
   TYPE_FIELDS (record_type) = fieldlist;
 
@@ -676,6 +734,7 @@ finish_record_type (record_type, fieldlist, has_rep)
     {
       tree field;
       int must_be_blkmode = 0;
+      int has_size = (TYPE_SIZE (record_type) != 0);
 
       TYPE_ALIGN (record_type) = MAX (BITS_PER_UNIT, TYPE_ALIGN (record_type));
       TYPE_MODE (record_type) = BLKmode;
@@ -715,6 +774,30 @@ finish_record_type (record_type, fieldlist, has_rep)
 	    must_be_blkmode = 1;
 	}
 
+      /* First round our size to a multiple of a byte.  */
+      TYPE_SIZE (record_type)
+	= round_up (TYPE_SIZE (record_type), BITS_PER_UNIT);
+
+      /* If a size was not specified, round the size we've computed
+	 to the required alignment of the type.  If a size was specified,
+	 see if it restricts the alignment.  */
+      if (! has_size)
+	{
+#ifdef ROUND_TYPE_SIZE
+	  TYPE_SIZE (record_type)
+	    = ROUND_TYPE_SIZE (record_type, TYPE_SIZE (record_type),
+			       TYPE_ALIGN (record_type));
+#else
+	  TYPE_SIZE (record_type) = round_up (TYPE_SIZE (record_type),
+					      TYPE_ALIGN (record_type));
+#endif
+	}
+      else
+	TYPE_ALIGN (record_type)
+	  = MIN (TYPE_ALIGN (record_type),
+		 (TREE_INT_CST_LOW (TYPE_SIZE (record_type))
+		  & (- TREE_INT_CST_LOW (TYPE_SIZE (record_type)))));
+
       if (! must_be_blkmode)
 	TYPE_MODE (record_type)
 	  = mode_for_size (TREE_INT_CST_LOW (TYPE_SIZE (record_type)),
@@ -738,7 +821,8 @@ finish_record_type (record_type, fieldlist, has_rep)
   else
     layout_type (record_type);
 
-  rest_of_type_compilation (record_type, global_bindings_p ());
+  if (! defer_debug)
+    rest_of_type_compilation (record_type, global_bindings_p ());
 }
 
 /* Return a FUNCTION_TYPE node. RETURN_TYPE is the type returned by the
@@ -747,15 +831,15 @@ finish_record_type (record_type, fieldlist, has_rep)
    PARM_DECL nodes that are the subprogram arguments.  CICO_LIST is the
    copy-in/copy-out list to be stored into TYPE_CICO_LIST.
    RETURNS_UNCONSTRAINED is nonzero if the function returns an unconstrained
-   object.  */
+   object.  RETURNS_BY_REF is nonzero if the function returns by reference.  */
 
 tree
 create_subprog_type (return_type, param_decl_list, cico_list,
-		     returns_unconstrained)
+		     returns_unconstrained, returns_by_ref)
      tree return_type;
      tree param_decl_list;
      tree cico_list;
-     int returns_unconstrained;
+     int returns_unconstrained, returns_by_ref;
 {
   /* A chain of TREE_LIST nodes whose TREE_VALUEs are the data type nodes of
      the subprogram formal parameters. This list is generated by traversing th
@@ -783,9 +867,10 @@ create_subprog_type (return_type, param_decl_list, cico_list,
 
   /* TYPE may have been shared since GCC hashes types.  If it has a CICO_LIST
      or the new type should, make a copy of TYPE.  Likewise for
-     RETURNS_UNCONSTRAINED.  */
+     RETURNS_UNCONSTRAINED and RETURNS_BY_REF.  */
   if (TYPE_CI_CO_LIST (type) != 0 || cico_list != 0
-      || TYPE_RETURNS_UNCONSTRAINED_P (type) != returns_unconstrained)
+      || TYPE_RETURNS_UNCONSTRAINED_P (type) != returns_unconstrained
+      || TYPE_RETURNS_BY_REF_P (type) != returns_by_ref)
     {
       push_obstacks (TYPE_OBSTACK (type), TYPE_OBSTACK (type));
       type = copy_node (type);
@@ -794,6 +879,7 @@ create_subprog_type (return_type, param_decl_list, cico_list,
 
   TYPE_CI_CO_LIST (type) = cico_list;
   TYPE_RETURNS_UNCONSTRAINED_P (type) = returns_unconstrained;
+  TYPE_RETURNS_BY_REF_P (type) = returns_by_ref;
   return type;
 }
 
@@ -847,14 +933,26 @@ create_index_type (min, max, index)
    string) and TYPE is a ..._TYPE node giving its data type.  */
 
 tree
-create_type_decl (type_name, type)
+create_type_decl (type_name, type, machine_attr)
      char *type_name;
      tree type;
+     tree machine_attr;
 {
   tree id_node   = type_name ? get_identifier (type_name): NULL_TREE;
   tree type_decl = build_decl (TYPE_DECL, id_node, type);
   enum tree_code code = TREE_CODE (type);
 
+  if (machine_attr)
+    {
+#ifdef DECL_STATIC_CONSTRUCTOR /* GCC 2.7.0  */
+      if (valid_machine_attribute (TREE_PURPOSE (machine_attr), NULL_TREE,
+				   type_decl, type))
+	;
+      else
+#endif
+	warning ("`%s' attribute directive ignored",
+		 IDENTIFIER_POINTER (TREE_PURPOSE (machine_attr)));
+    }
   /* Add this decl to the current binding level.  */
   type_decl = pushdecl (type_decl);
 
@@ -865,16 +963,12 @@ create_type_decl (type_name, type)
   if (code != UNCONSTRAINED_ARRAY_TYPE && code != ENUMERAL_TYPE 
       && code != RECORD_TYPE && ! TYPE_IS_DUMMY_P (type))
     rest_of_decl_compilation (type_decl, NULL, global_bindings_p (), 0);
+  else
+    DECL_IGNORED_P (type_decl) = 1;
 
   return type_decl;
 }
 
-/* This listhead is used to record any global objects that need elaboration.
-   TREE_PURPOSE is the variable to be elaborated and TREE_VALUE is the
-   initial value to assign.  */
-
-static tree pending_elaborations;
-
 /* Returns a GCC VAR_DECL node. VAR_NAME gives the name of the variable (a
    character string). ASM_NAME is its assembler name (if provided).  TYPE is
    its data type (a GCC ..._TYPE node).  VAR_INIT is the GCC tree for an
@@ -916,7 +1010,10 @@ create_var_decl (var_name, asm_name, type, var_init, size, align,
 		   && TREE_CODE (TYPE_SIZE (type)) == INTEGER_CST
 		   && TREE_INT_CST_HIGH (TYPE_SIZE (type)) == 0
 		   && (TREE_INT_CST_LOW (TYPE_SIZE (type))
-		       <= GET_MODE_BITSIZE (DCmode)))
+		       <= GET_MODE_BITSIZE (DCmode))
+		   /* For the moment, don't do this for records
+		      due to the Assignment_OK flag.  */
+		   && ! AGGREGATE_TYPE_P (type))
 		  ? CONST_DECL : VAR_DECL, id_node, type);
 
   /* If this is external, throw away any initializations unless this is a
@@ -938,16 +1035,15 @@ create_var_decl (var_name, asm_name, type, var_init, size, align,
   TREE_READONLY (var_decl) = const_flag;
   DECL_EXTERNAL (var_decl) = extern_flag;
   TREE_PUBLIC   (var_decl) = public_flag || extern_flag;
-  TREE_STATIC   (var_decl)
-    = (global_bindings_p () ? !extern_flag
-       : static_flag || TYPE_VOLATILE (type));
   TREE_CONSTANT (var_decl) = TREE_CODE (var_decl) == CONST_DECL;
   TREE_THIS_VOLATILE (var_decl) = TREE_SIDE_EFFECTS (var_decl)
     = TYPE_VOLATILE (type);
 
   /* At the global binding level we need to allocate static storage for the
      variable if and only if its not external. If we are not at the top level
-     we always allocate automatic storage. */
+     we allocate automatic storage unless requested not to.  */
+  TREE_STATIC (var_decl) = global_bindings_p () ? !extern_flag : static_flag;
+
   if (asm_name)
     DECL_ASSEMBLER_NAME (var_decl) = get_identifier (asm_name);
 
@@ -969,6 +1065,11 @@ create_var_decl (var_name, asm_name, type, var_init, size, align,
   var_decl = pushdecl (var_decl);
   expand_decl (var_decl);
   expand_decl_init (var_decl);
+
+  /* If this is volatile, force it into memory.  */
+  if (TREE_SIDE_EFFECTS (var_decl))
+    mark_addressable (var_decl);
+
   if (TREE_CODE (var_decl) != CONST_DECL)
     rest_of_decl_compilation (var_decl, 0, global_bindings_p (), 0);
 
@@ -978,7 +1079,10 @@ create_var_decl (var_name, asm_name, type, var_init, size, align,
 /* Returns a FIELD_DECL node. FIELD_NAME the field name, FIELD_TYPE is its
    type, and RECORD_TYPE is the type of the parent.  PACKED is nonzero if
    this field is in a record type with a "pragma pack".  If SIZE is nonnegative
-   it is the specified size for this field and POS is the bit position.  */
+   it is the specified size for this field and POS is the bit position.  
+   If SIZE is the special value -2, it means we should not replace a
+   discriminated size with the maximum value; this is used in laying
+   out nested variants.  */
 
 tree
 create_field_decl (field_name, field_type, record_type, packed, size, pos)
@@ -992,26 +1096,35 @@ create_field_decl (field_name, field_type, record_type, packed, size, pos)
   tree field_decl = build_decl (FIELD_DECL, field_id, field_type);
 
   DECL_CONTEXT (field_decl) = record_type;
+  TREE_READONLY (field_decl) = TREE_READONLY (field_type);
 
   /* If we have a specified size and it is not the same as the size
      of the mode or if the position does not correspond to the
      proper alignment, make this a bit field.  Likewise for a BLKmode
-     field not byte aligned.  */
+     field not properly aligned.  */
   if (size >= 0)
     {
       if (((INTEGRAL_TYPE_P (field_type)
 	    && (size != GET_MODE_BITSIZE (TYPE_MODE (field_type))
 		|| pos % GET_MODE_ALIGNMENT (TYPE_MODE (field_type) != 0)))
 	  || (TYPE_MODE (field_type) == BLKmode
-	      && pos % BITS_PER_UNIT != 0)))
+	      && pos % MAX (TYPE_ALIGN (field_type), BITS_PER_UNIT))))
 	{
 	  DECL_BIT_FIELD (field_decl) = 1;
 	  DECL_PACKED (field_decl) = 1;
 	}
+      else if (TREE_CODE (TYPE_SIZE (field_type)) != INTEGER_CST
+	       || size != TREE_INT_CST_LOW (TYPE_SIZE (field_type)))
+	DECL_BIT_FIELD (field_decl) = 1;
 
       DECL_FIELD_SIZE (field_decl) = size;
       DECL_FIELD_BITPOS (field_decl) = size_int (pos);
       layout_decl (field_decl, pos);
+
+      /* ??? Only leave it considered a bit field if we need to.  Set it
+	 above to have layout_decl use our size.  We need to clean up
+	 this mess one of these days.  */
+      DECL_BIT_FIELD (field_decl) = DECL_PACKED (field_decl);
     }
 
   /* If this is to be packed and it is an integral type make a bit field whose
@@ -1030,7 +1143,7 @@ create_field_decl (field_name, field_type, record_type, packed, size, pos)
 
   /* If the type is a RECORD_TYPE and its size depends on a discriminant,
      the size to be used for the object is the maximum possible size.  */
-  if (TREE_CODE (field_type) == RECORD_TYPE
+  if (size != -2 && TREE_CODE (field_type) == RECORD_TYPE
       && ! TREE_CONSTANT (TYPE_SIZE (field_type))
       && contains_placeholder_except_p (TYPE_SIZE (field_type), record_type))
     DECL_SIZE (field_decl) = max_size (TYPE_SIZE (field_type), 1);
@@ -1039,17 +1152,29 @@ create_field_decl (field_name, field_type, record_type, packed, size, pos)
 }
 
 /* Returns a PARM_DECL node. PARAM_NAME is the name of the parameter,
-   PARAM_TYPE is its type.  */
+   PARAM_TYPE is its type.  READONLY is nonzero if the parameter is
+   readonly (either an IN parameter or an address of a pass-by-ref
+   parameter). */
 
 tree
-create_param_decl (param_name, param_type)
+create_param_decl (param_name, param_type, readonly)
      char *param_name;
      tree param_type;
+     int readonly;
 {
   tree param_id   = get_identifier (param_name);
   tree param_decl = build_decl (PARM_DECL, param_id, param_type);
 
+#ifdef PROMOTE_PROTOTYPES
+  if ((TREE_CODE (param_type) == INTEGER_TYPE
+       || TREE_CODE (param_type) == ENUMERAL_TYPE)
+      && TYPE_PRECISION (param_type) < TYPE_PRECISION (integer_type_node))
+    param_type = integer_type_node;
+#endif
+
   DECL_ARG_TYPE (param_decl) = param_type;
+
+  TREE_READONLY (param_decl) = readonly;
   return param_decl;
 }
 
@@ -1060,7 +1185,8 @@ add_pending_elaborations (var_decl, var_init)
      tree var_decl;
      tree var_init;
 {
-  pending_elaborations = tree_cons (var_decl, var_init, pending_elaborations);
+  pending_elaborations
+    = chainon (pending_elaborations, build_tree_list (var_decl, var_init));
 }
 
 /* Obtain any pending elaborations and clear the old list.  */
@@ -1070,12 +1196,60 @@ get_pending_elaborations ()
 {
   /* Each thing added to the list went on the end; we want it on the
      beginning.  */
-  tree result = nreverse (pending_elaborations);
+  tree result = TREE_CHAIN (pending_elaborations);
 
-  pending_elaborations = 0;
+  TREE_CHAIN (pending_elaborations) = 0;
   return result;
 }
 
+/* Save a copy of the current pending elaboration list and make a new
+   one.  */
+
+void
+push_pending_elaborations ()
+{
+  struct e_stack *p = (struct e_stack *) oballoc (sizeof (struct e_stack));
+
+  p->next = elist_stack;
+  p->elab_list = pending_elaborations;
+  elist_stack = p;
+  pending_elaborations = build_tree_list (NULL_TREE, NULL_TREE);
+}
+
+/* Pop the stack of pending elaborations.  */
+
+void
+pop_pending_elaborations ()
+{
+  pending_elaborations = elist_stack->elab_list;
+  elist_stack = elist_stack->next;
+}
+
+/* Return the current position in pending_elaborations so we can insert
+   elaborations after that point.  */
+
+tree
+get_elaboration_location ()
+{
+  return pending_elaborations;
+}
+
+/* Insert the current elaborations after ELAB, which is in some elaboration
+   list.  */
+
+void
+insert_elaboration_list (elab)
+     tree elab;
+{
+  tree next = TREE_CHAIN (elab);
+
+  if (TREE_CHAIN (pending_elaborations))
+    {
+      TREE_CHAIN (elab) = TREE_CHAIN (pending_elaborations);
+      TREE_CHAIN (tree_last (pending_elaborations)) = next;
+    }
+}
+
 /* Returns a LABEL_DECL node for LABEL_NAME.  */
 
 tree
@@ -1127,14 +1301,16 @@ create_subprog_decl (subprog_name, asm_name, subprog_type, param_decl_list,
   DECL_RESULT (subprog_decl)    = build_decl (RESULT_DECL, 0, return_type);
 
   if (machine_attr)
-#if defined (VALID_MACHINE_DECL_ATTRIBUTE) \
-    || defined (VALID_MACHINE_TYPE_ATTRIBUTE)
-    if (valid_machine_attribute (machine_attr, subprog_decl, subprog_type))
-      ;
-    else
+    {
+#ifdef DECL_STATIC_CONSTRUCTOR /* GCC 2.7.0  */
+      if (valid_machine_attribute (TREE_PURPOSE (machine_attr), NULL_TREE,
+				   subprog_decl, subprog_type))
+	;
+      else
 #endif
-      warning ("`%s' attribute directive ignored",
-               IDENTIFIER_POINTER (TREE_PURPOSE (machine_attr)));
+	warning ("`%s' attribute directive ignored",
+		 IDENTIFIER_POINTER (TREE_PURPOSE (machine_attr)));
+    }
 
   if (asm_name)
     DECL_ASSEMBLER_NAME (subprog_decl) = get_identifier (asm_name);
@@ -1176,9 +1352,6 @@ begin_subprog_body (subprog_decl)
      adequate BLOCK.  */
   DECL_INITIAL (subprog_decl)  = error_mark_node;
 
-  /* This is a definition, not a reference. So clear DECL_EXTERNAL. */
-  DECL_EXTERNAL (subprog_decl) = 0;
-
   /* This function exists in static storage. This does not mean `static' in
      the C sense!  */
   TREE_STATIC (subprog_decl)   = 1;
@@ -1232,6 +1405,15 @@ end_subprog_body (void)
   expand_function_end (input_filename, lineno, 0);
   rest_of_compilation (current_function_decl);
 
+  if (DECL_SAVED_INSNS (current_function_decl) == 0)
+    {
+      /* Stop pointing to the local nodes about to be freed.  */
+      /* But DECL_INITIAL must remain nonzero so we know this
+	 was an actual function definition.  */
+      DECL_INITIAL (current_function_decl) = error_mark_node;
+      DECL_ARGUMENTS (current_function_decl) = 0;
+    }
+
   /* If we are not at the bottom of the function nesting stack, pop up to
      the containing function.  Otherwise show we aren't in any function
      and switch back to permanent allocation.  */
@@ -1495,7 +1677,6 @@ build_template (template_type, array_type, expr)
 {
   tree template_elts = NULL_TREE;
   tree field;
-  tree min, max;
 
   /* First make the list for a CONSTRUCTOR for the template.   Go down the
      field list of the template instead of the type chain because this
@@ -1506,11 +1687,27 @@ build_template (template_type, array_type, expr)
        array_type = TREE_TYPE (array_type),
        field = TREE_CHAIN (TREE_CHAIN (field)))
     {
-      if (TREE_CODE (array_type) != ARRAY_TYPE)
+      tree bounds, min, max;
+
+      /* If we have an array type, get the bounds from there.  If not
+	 and expr is a PARM_DECL with DECL_BY_COMPONENT_PTR_P,
+	 use the bounds of the field in the template.  This will give
+	 us a maximum range.  */
+      if (TREE_CODE (array_type) == ARRAY_TYPE)
+	{
+	  if (TYPE_ACTUAL_BOUNDS (array_type) != 0)
+	    bounds = TYPE_ACTUAL_BOUNDS (array_type);
+	  else
+	    bounds = TYPE_INDEX_TYPE (TYPE_DOMAIN (array_type));
+	}
+      else if (expr != 0 && TREE_CODE (expr) == PARM_DECL
+	       && DECL_BY_COMPONENT_PTR_P (expr))
+	bounds = TREE_TYPE (field);
+      else
 	gigi_abort (411);
 
-      min = TYPE_MIN_VALUE (TYPE_INDEX_TYPE (TYPE_DOMAIN (array_type)));
-      max = TYPE_MAX_VALUE (TYPE_INDEX_TYPE (TYPE_DOMAIN (array_type)));
+      min = convert (TREE_TYPE (TREE_CHAIN (field)), TYPE_MIN_VALUE (bounds));
+      max = convert (TREE_TYPE (field), TYPE_MAX_VALUE (bounds));
 
       /* If either MIN or MAX involve a PLACEHOLDER_EXPR, we must
 	 surround them with a WITH_RECORD_EXPR giving EXPR as the
@@ -1527,6 +1724,34 @@ build_template (template_type, array_type, expr)
   return build_constructor (template_type, nreverse (template_elts));
 }
 
+/* Build a type to be used to represent an aliased object whose nominal
+   type is an unconstrained array.  This consists of a RECORD_TYPE containing
+   a field of TEMPLATE_TYPE and a field of OBJECT_TYPE, which is an
+   ARRAY_TYPE.  If ARRAY_TYPE is that of the unconstrained array, this
+   is used to represent an arbitrary unconstrained object.  Use NAME
+   as the name of the record.  */
+
+tree
+build_unc_object_type (template_type, object_type, name)
+     tree template_type;
+     tree object_type;
+     char *name;
+{
+  tree type = make_node (RECORD_TYPE);
+  tree template_field
+    = create_field_decl ("BOUNDS", template_type, type, 0, -1, 0);
+  tree array_field = create_field_decl ("ARRAY", object_type, type, 0, -1, 0);
+
+  TYPE_NAME (type) = get_identifier (name);
+  finish_record_type (type,
+		      chainon (chainon (NULL_TREE, template_field),
+			       array_field),
+		      0, 0);
+  TYPE_CONTAINS_TEMPLATE_P (type) = 1;
+
+  return type;
+}
+
 /* Update anything previously pointing to OLD_TYPE to point to NEW_TYPE.  In
    the normal case this is just two adjustments, but we have more to do
    if NEW is an UNCONSTRAINED_ARRAY_TYPE.  */
@@ -1576,19 +1801,24 @@ update_pointer_to (old_type, new_type)
 
       TYPE_UNCONSTRAINED_ARRAY (ptr) = new_type;
       TYPE_POINTER_TO (new_type) = TREE_TYPE (new_type) = ptr;
+      rest_of_type_compilation (ptr, global_bindings_p ());
+
+      update_pointer_to (TYPE_OBJECT_RECORD_TYPE (old_type),
+			 TYPE_OBJECT_RECORD_TYPE (new_type));
     }
 }
 
-/* Convert a pointer to a constrained array into a pointer to an unconstrained
-   array.  This involves making a template.  */
+/* Convert a pointer to a constrained array into a pointer to a fat
+   pointer.  This involves making or finding a template.  */
 
-tree
-convert_to_unconstrained (type, expr)
+static tree
+convert_to_fat_pointer (type, expr)
      tree type;
      tree expr;
 {
   tree template_type = TREE_TYPE (TREE_TYPE (TREE_CHAIN (TYPE_FIELDS (type))));
-  tree template_cons, template_addr;
+  tree template, template_addr;
+  tree etype = TREE_TYPE (expr);
 
   /* If EXPR is a constant of zero, we make a fat pointer that has a null
      pointer to the template and array.  */
@@ -1603,25 +1833,26 @@ convert_to_unconstrained (type, expr)
 					expr),
 			       NULL_TREE)));
 
-  /* Build the constructor for the template.  If EXPR is a pointer to a
-     COMPONENT_REF from a TYPE_CONTAINS_TEMPLATE_P, get the template from the
-     underlying type.  In that case, we don't have to worry about
-     allocation.  */
-  if (TREE_CODE (expr) == ADDR_EXPR
-      && TREE_CODE (TREE_OPERAND (expr, 0)) == COMPONENT_REF
-      && (TYPE_CONTAINS_TEMPLATE_P
-	  (TREE_TYPE (TREE_OPERAND (TREE_OPERAND (expr, 0), 0)))))
-    {
-      tree underlying = TREE_OPERAND (TREE_OPERAND (expr, 0), 0);
-      tree temp_field = TYPE_FIELDS (TREE_TYPE (underlying));
+  /* If EXPR's type is a pointer to a TYPE_CONTAINS_TEMPLATE_P record, get
+     the template and data from the record.  */
 
-      template_cons = build_component_ref (underlying, NULL_TREE, temp_field);
+  else if (TREE_CODE (etype) == POINTER_TYPE
+	   && TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (etype)))
+    {
+      tree fields = TYPE_FIELDS (TREE_TYPE (etype));
+
+      expr = save_expr (expr);
+      expr = build1 (INDIRECT_REF, TREE_TYPE (etype), expr);
+      template = build_component_ref (expr, NULL_TREE, fields);
+      expr = build_unary_op (ADDR_EXPR, NULL_TREE,
+			     build_component_ref (expr, NULL_TREE,
+						  TREE_CHAIN (fields)));
     }
   else
-    template_cons
-      = build_template (template_type, TREE_TYPE (TREE_TYPE (expr)), expr);
+    /* Otherwise, build the constructor for the template.  */
+    template = build_template (template_type, TREE_TYPE (etype), expr);
 
-  template_addr = build_unary_op (ADDR_EXPR, NULL_TREE, template_cons);
+  template_addr = build_unary_op (ADDR_EXPR, NULL_TREE, template);
 
   /* The result is a CONSTRUCTOR for the fat pointer.  */
   return
@@ -1631,6 +1862,28 @@ convert_to_unconstrained (type, expr)
 					     template_addr, NULL_TREE)));
 }
 
+/* Convert to a thin pointer type, TYPE.  The only thing we know how to convert
+   is something that is a fat pointer, so convert to it first if it EXPR
+   is not already a fat pointer.  */
+
+static tree
+convert_to_thin_pointer (type, expr)
+     tree type;
+     tree expr;
+{
+  if (! TYPE_FAT_POINTER_P (TREE_TYPE (expr)))
+    expr
+      = convert_to_fat_pointer
+	(TREE_TYPE (TYPE_UNCONSTRAINED_ARRAY (TREE_TYPE (type))), expr);
+
+  /* We get the pointer to the data and use a NOP_EXPR to make it the
+     proper GCC type.  */
+  expr = build_component_ref (expr, NULL_TREE, TYPE_FIELDS (TREE_TYPE (expr)));
+  expr = build1 (NOP_EXPR, type, expr);
+
+  return expr;
+}
+
 /* Create an expression whose value is that of EXPR,
    converted to type TYPE.  The TREE_TYPE of the value
    is always TYPE.  This function implements all reasonable
@@ -1663,14 +1916,17 @@ convert (type, expr)
       return expr;
 
     case STRING_CST:
+    case CONSTRUCTOR:
       /* If we are converting a STRING_CST to another constrained array type,
-	 just make a new one in the proper type.  */
-      if (code != ARRAY_TYPE)
-	break;
-
-      expr = copy_node (expr);
-      TREE_TYPE (expr) = type;
-      return expr;
+	 just make a new one in the proper type.  Likewise for a
+	 CONSTRUCTOR.  */
+      if (code == ecode && AGGREGATE_TYPE_P (etype))
+	{
+	  expr = copy_node (expr);
+	  TREE_TYPE (expr) = type;
+	  return expr;
+	}
+      break;
 
     case UNCONSTRAINED_ARRAY_REF:
       /* Convert this to the type of the inner array by getting the address of
@@ -1682,13 +1938,9 @@ convert (type, expr)
       break;
     }
 
-  /* Check for converting a pointer to a constrained array into a pointer to
-     a constrained array.  */
-  if (TYPE_FAT_POINTER_P (type)
-      && (integer_zerop (expr)
-	  || (ecode == POINTER_TYPE
-	      && TREE_CODE (TREE_TYPE (etype)) == ARRAY_TYPE)))
-    return convert_to_unconstrained (type, expr);
+  /* Check for converting to a pointer to an unconstrained array.  */
+  if (TYPE_FAT_POINTER_P (type) && ! TYPE_FAT_POINTER_P (etype))
+    return convert_to_fat_pointer (type, expr);
 
   if (TYPE_MAIN_VARIANT (type) == TYPE_MAIN_VARIANT (etype)
       || (code == INTEGER_CST && ecode == INTEGER_CST
@@ -1729,9 +1981,36 @@ convert (type, expr)
       return fold (convert_to_integer (type, expr));
 
     case POINTER_TYPE:
+      /* If converting between two pointers to records denoting
+	 both a template and type, adjust if needed to account
+	 for any differing offsets, since one might be negative.  */
+      if (ecode == POINTER_TYPE
+	  && TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (etype))
+	  && TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (type)))
+	{
+	  tree bit_diff
+	    = size_binop (MINUS_EXPR,
+			  DECL_FIELD_BITPOS (TYPE_FIELDS (TREE_TYPE (etype))),
+			  DECL_FIELD_BITPOS (TYPE_FIELDS (TREE_TYPE (type))));
+	  tree byte_diff = size_binop (CEIL_DIV_EXPR, bit_diff,
+				       size_int (BITS_PER_UNIT));
+
+	  expr = build1 (NOP_EXPR, type, expr);
+	  if (integer_zerop (byte_diff))
+	    return expr;
+
+	  return build_binary_op (PLUS_EXPR, type, expr,
+				  convert_to_pointer (type, byte_diff));
+	}
+
+      /* If converting to a thin pointer, handle specially.  */
+      if (TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (type))
+	  && TYPE_UNCONSTRAINED_ARRAY (TREE_TYPE (type)) != 0)
+	return convert_to_thin_pointer (type, expr);
+
       /* If converting fat pointer to normal pointer, get the pointer to the
 	 array and then convert it.  */
-      if (TYPE_FAT_POINTER_P (etype))
+      else if (TYPE_FAT_POINTER_P (etype))
 	expr = build_component_ref (expr, get_identifier ("P_ARRAY"),
 				    NULL_TREE);
 
@@ -1759,14 +2038,15 @@ convert (type, expr)
 
     case UNCONSTRAINED_ARRAY_TYPE:
       /* If EXPR is a constrained array, take its address, convert it to a
-	 fat pointer, and then dereference it.  */
-      if (ecode == ARRAY_TYPE)
+	 fat pointer, and then dereference it.  Likewise if EXPR is a
+	 record containing both a template and a constrained array.  */
+      if (ecode == ARRAY_TYPE || TYPE_CONTAINS_TEMPLATE_P (etype))
 	return
 	  build_unary_op
 	    (INDIRECT_REF, NULL_TREE,
-	     convert_to_unconstrained (TREE_TYPE (type),
-				       build_unary_op (ADDR_EXPR,
-						       NULL_TREE, expr)));
+	     convert_to_fat_pointer (TREE_TYPE (type),
+				     build_unary_op (ADDR_EXPR,
+						     NULL_TREE, expr)));
 
       /* Do something very similar for converting one unconstrained
 	 array to another.  */
