@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.73 $                             */
+/*                            $Revision: 1.90 $                             */
 /*                                                                          */
-/*           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          */
+/*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -18,7 +18,11 @@
 /* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
 /* for  more details.  You should have  received  a copy of the GNU General */
 /* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
@@ -88,6 +92,9 @@ extern int force_global;
    front-end.  */
 #define TYPE_EXTRA_SUBTYPE_P(NODE) TYPE_LANG_FLAG_2 (NODE)
 
+/* Nonzero for composite types if this is a by-reference type.  */
+#define TYPE_BY_REFERENCE_P(NODE) TYPE_LANG_FLAG_2 (NODE)
+
 /* Nonzero if this is the type for an object whose type includes
    its template in addition to its value.  */
 #define TYPE_CONTAINS_TEMPLATE_P(NODE) TYPE_LANG_FLAG_3 (NODE)
@@ -95,6 +102,9 @@ extern int force_global;
 /* For ARRAY_TYPEs, nonzero if the array type has Convention_Fortran.  */
 #define TYPE_CONVENTION_FORTRAN_P(NODE) TYPE_LANG_FLAG_4 (NODE)
 
+/* For FUNCTION_TYPEs, nonzero if the function returns by reference.  */
+#define TYPE_RETURNS_BY_REF_P(NODE) TYPE_LANG_FLAG_4 (NODE)
+
 /* For VOID_TYPE and RECORD_TYPE, nonzero if this is a dummy type,
    make to correspond to a private or incomplete type.  */
 #define TYPE_DUMMY_P(NODE) TYPE_LANG_FLAG_4 (NODE)
@@ -104,6 +114,14 @@ extern int force_global;
   ((TREE_CODE (NODE) == VOID_TYPE || TREE_CODE (NODE) == RECORD_TYPE) \
    && TYPE_DUMMY_P (NODE))
 
+/* Nonzero if this corresponds to a type where alignment is guaranteed
+   by other mechanisms (a tagged or packed type).  */
+#define TYPE_ALIGN_OK_P(NODE) TYPE_LANG_FLAG_5 (NODE)
+
+/* For RECORD_TYPE, nonzero if this was made for the component type
+   of an ARRAY_TYPE where we had to use max_size.  */
+#define TYPE_COMPONENT_MAX_TYPE_P(NODE) TYPE_LANG_FLAG_6 (NODE)
+
 /* This field is only defined for FUNCTION_TYPE nodes. If the Ada
    subprogram contains no parameters passed by copy in/copy out then this
    field is 0. Otherwise it points to a list of nodes used to specify the
@@ -121,13 +139,23 @@ extern int force_global;
 #define TYPE_INDEX_TYPE(NODE)	(tree) TYPE_LANG_SPECIFIC (NODE)
 
 /* For a RECORD_TYPE that is a fat pointer, point to the type for the
-   unconstrained object.  */
+   unconstrained object.  Likewise for a RECORD_TYPE that is pointed
+   to by a thin pointer.  */
 #define TYPE_UNCONSTRAINED_ARRAY(NODE)  (tree) TYPE_LANG_SPECIFIC (NODE)
 
+/* For an ARRAY_TYPE, this is the index type that should be used when
+   the actual bounds are required for a template.  This is used in
+   the case of packed arrays.  */
+#define TYPE_ACTUAL_BOUNDS(NODE)   (tree) TYPE_LANG_SPECIFIC (NODE)
+
 /* For a RECORD_TYPE that is not a fat pointer, point to the original subtype
    for the _parent field, if there was one.  */
 #define TYPE_PARENT_SUBTYPE(NODE)  (tree) TYPE_LANG_SPECIFIC (NODE)
 
+/* In an UNCONSTRAINED_ARRAY_TYPE, points to the record containing both
+   the template and object.  */
+#define TYPE_OBJECT_RECORD_TYPE(NODE) TYPE_MIN_VALUE (NODE)
+
 /* Nonzero in a FIELD_DECL that represents a discriminant.  */
 #define DECL_DISCRIMINANT_P(NODE) DECL_LANG_FLAG_0 (NODE)
 
@@ -149,6 +177,10 @@ extern int force_global;
 /* Nonzero in a FIELD_DECL that is a dummy built for a variant part.  */
 #define DECL_FOR_VARIANT_P(NODE) DECL_LANG_FLAG_5 (NODE)
 
+/* Nonzero if this is a decl for a pointer that points to something which
+   is readonly.  Used mostly for fat pointers.  */
+#define DECL_POINTS_TO_READONLY_P(NODE) DECL_LANG_FLAG_6 (NODE)
+
 /* Nonzero in a TRANSFORM_EXPR if we are to take its address.  */
 #define TREE_TRANSFORM_ADDR(NODE) TREE_STATIC (NODE)
 
@@ -195,6 +227,9 @@ extern tree malloc_decl;
 /* Likewise for freeing memory.  */
 extern tree free_decl;
 
+/* Likewise for setting the current exception occurrence */
+extern tree set_except_occ_decl;
+
 /* Types and decls used by our temporary exception mechanism.  See
    init_decl_processing for details.  */
 extern tree jmpbuf_type;
@@ -207,7 +242,7 @@ extern tree raise_nodefer_decl;
 extern tree setjmp_decl;
 extern tree longjmp_decl;
 extern tree raise_constraint_error_decl;
-extern tree unchecked_union_node;
+extern tree raise_program_error_decl;
 
 /* Routines expected by the gcc back-end. They must have exactly the same
    prototype and names as below.  */
@@ -320,8 +355,10 @@ extern void init_gnat_to_gnu		PROTO((void));
 /* Given a record type (RECORD_TYPE) and a chain of FIELD_DECL
    nodes (FIELDLIST), finish constructing the record or union type. 
    If HAS_REP is nonzero, this record has a rep clause; don't call
-   layout_type but merely set the size and alignment ourselves.  */
-extern void finish_record_type		PROTO((tree, tree, int));
+   layout_type but merely set the size and alignment ourselves.
+   If DEFER_DEBUG is nonzero, do not call the debugging routines
+   on this type; it will be done later. */
+extern void finish_record_type		PROTO((tree, tree, int, int));
 
 /* Returns a FUNCTION_TYPE node. RETURN_TYPE is the type returned by the
    subprogram. If it is void_type_node, then we are dealing with a procedure,
@@ -329,8 +366,8 @@ extern void finish_record_type		PROTO((tree, tree, int));
    PARM_DECL nodes that are the subprogram arguments.  CICO_LIST is the
    copy-in/copy-out list to be stored into TYPE_CI_CO_LIST. 
    RETURNS_UNCONSTRAINED is nonzero if the function returns an unconstrained
-   object.  */
-extern tree create_subprog_type		PROTO((tree, tree, tree, int));
+   object.  RETURNS_BY_REF is nonzero if the function returns by reference.  */
+extern tree create_subprog_type		PROTO((tree, tree, tree, int, int));
 
 /* Return a copy of TYPE, in the same obstack as it was, but safe to modify
    in any way.  */
@@ -343,7 +380,7 @@ extern tree create_index_type		PROTO((tree, tree, tree));
 
 /* Returns a TYPE_DECL node. TYPE_NAME gives the name of the type (a character
    string) and TYPE is a ..._TYPE node giving its data type.  */
-extern tree create_type_decl		PROTO((char *, tree));
+extern tree create_type_decl		PROTO((char *, tree, tree));
 
 /* Returns a GCC VAR_DECL node. VAR_NAME gives the name of the variable (a
    character string). ASM_NAME is its assembler name (if provided).  TYPE is
@@ -368,19 +405,38 @@ extern tree create_var_decl	PROTO((char *, char *, tree, tree, tree, int,
 /* Obtain any pending elaborations and clear the old list.  */
 extern tree get_pending_elaborations PROTO((void));
 
-/* Add some pending elaborations to the current list.  */
-extern void add_pending_elaborations PROTO ((tree, tree));
+/* Save a copy of the current pending elaboration list and make a new
+   one.  */
+extern void push_pending_elaborations PROTO((void));
 
+/* Pop the stack of pending elaborations.  */
+extern void pop_pending_elaborations PROTO((void));
+
+/* Return the current position in pending_elaborations so we can insert
+   elaborations after that point.  */
+extern tree get_elaboration_location PROTO((void));
+
+/* Insert the current elaborations after ELAB, which is in some elaboration
+   list.  */
+extern void insert_elaboration_list PROTO((tree));
+
+/* Add some pending elaborations to the current list.  */
+extern void add_pending_elaborations PROTO((tree, tree));
 
 /* Returns a FIELD_DECL node. FIELD_NAME the field name, FIELD_TYPE is its
    type, and RECORD_TYPE is the type of the parent.  PACKED is nonzero if
    this field is in a record type with a "pragma pack".  If SIZE is nonnegative
-   it is the specified size for this field and POS is the bit position.  */
+   it is the specified size for this field and POS is the bit position.  
+   If SIZE is the special value -2, it means we should not replace a
+   discriminated size with the maximum value; this is used in laying
+   out nested variants.  */
 extern tree create_field_decl	PROTO((char *, tree, tree, int, int, int));
 
 /* Returns a PARM_DECL node. PARAM_NAME is the name of the parameter,
-   PARAM_TYPE is its type.  */
-extern tree create_param_decl	PROTO((char *, tree));
+   PARAM_TYPE is its type.  READONLY is nonzero if the parameter is
+   readonly (either an IN parameter or an address of a pass-by-ref
+   parameter). */
+extern tree create_param_decl	PROTO((char *, tree, int));
 
 /* Returns a FUNCTION_DECL node.  SUBPROG_NAME is the name of the subprogram,
    ASM_NAME is its assembler name, SUBPROG_TYPE is its type (a FUNCTION_TYPE
@@ -409,15 +465,19 @@ extern void end_subprog_body	PROTO((void));
    Return a constructor for the template.  */
 extern tree build_template	PROTO((tree, tree, tree));
 
+/* Build a type to be used to represent an aliased object whose nominal
+   type is an unconstrained array.  This consists of a RECORD_TYPE containing
+   a field of TEMPLATE_TYPE and a field of OBJECT_TYPE, which is an
+   ARRAY_TYPE.  If ARRAY_TYPE is that of the unconstrained array, this
+   is used to represent an arbitrary unconstrained object.  Use NAME
+   as the name of the record.  */
+extern tree build_unc_object_type  PROTO((tree, tree, char *));
+
 /* Update anything previously pointing to OLD_TYPE to point to NEW_TYPE.  In
    the normal case this is just two adjustments, but we have more to do
    if NEW is an UNCONSTRAINED_ARRAY_TYPE.  */
 extern void update_pointer_to	PROTO((tree, tree));
 
-/* Convert a pointer to a constrained array into a pointer to an unconstrained
-   array.  This involves making a template.  */
-extern tree convert_to_unconstrained PROTO((tree, tree));
-
 /* EXP is an expression for the size of an object.  If this size contains
    discriminant references, replace them with the maximum (if MAX_P) or
    minimum (if ! MAX_P) possible value of the discriminant.  */
