@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                            $Revision: 1.91 $                             */
+/*                            $Revision: 1.116 $                            */
 /*                                                                          */
-/*           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          */
+/*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -18,7 +18,11 @@
 /* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
 /* for  more details.  You should have  received  a copy of the GNU General */
 /* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
@@ -41,6 +45,11 @@
 #include "a-trans3.h"
 #include "a-trans4.h"
 #include "a-misc.h"
+
+static tree find_common_type		PROTO((tree, tree));
+static tree compare_arrays		PROTO((tree, tree, tree));
+static tree nonbinary_modular_operation	PROTO((enum tree_code, tree,
+					       tree, tree));
 
 /* Prepare expr to be an argument of a TRUTH_NOT_EXPR or other logical
    operation.
@@ -98,13 +107,28 @@ get_base_type (type)
 
   return type;
 }
+
+/* Likewise, but only return types known to the Ada source.  */
+tree
+get_ada_base_type (type)
+     tree type;
+{
+  while (TREE_TYPE (type) != 0
+	 && (TREE_CODE (type) == INTEGER_TYPE
+	     || TREE_CODE (type) == REAL_TYPE)
+	 && ! TYPE_EXTRA_SUBTYPE_P (type))
+    type = TREE_TYPE (type);
+
+  return type;
+}
 
-/* We have a comparison or assignment operation on two array types, T1 and T2.
+/* We have a comparison or assignment operation on two types, T1 and T2,
+   which are both either array types or both record types.
    Return the type that both operands should be converted to, if any.
    Otherwise return zero.  */
 
 static tree
-find_common_array_type (t1, t2)
+find_common_type (t1, t2)
      tree t1, t2;
 {
   /* If either type is non-BLKmode, use it.  Note that we know that we will
@@ -131,9 +155,10 @@ find_common_array_type (t1, t2)
    A1 and A2, two objects of ARRAY_TYPE.  The returned expression should
    be of type RESULT_TYPE
 
-   Two arrays are equal if the lengths in each dimension are equal
-   and the data is equal.  We perform the length tests in as efficient
-   a manner as possible.  */
+   Two arrays are equal in one of two ways: (1) if both have zero length
+   in some dimension (not necessarily the same dimension) or (2) if the
+   lengths in each dimension are equal and the data is equal.  We perform the
+   length tests in as efficient a manner as possible.  */
 
 static tree
 compare_arrays (result_type, a1, a2)
@@ -143,6 +168,8 @@ compare_arrays (result_type, a1, a2)
   tree t1 = TREE_TYPE (a1);
   tree t2 = TREE_TYPE (a2);
   tree result = convert (result_type, integer_one_node);
+  tree a1_is_null = convert (result_type, integer_zero_node);
+  tree a2_is_null = convert (result_type, integer_zero_node);
   int length_zero_p = 0;
 
   /* Process each dimension separately and compare the lengths.  If any
@@ -157,6 +184,7 @@ compare_arrays (result_type, a1, a2)
       tree bt = get_base_type (TREE_TYPE (lb1));
       tree length1 = fold (build (MINUS_EXPR, bt, ub1, lb1));
       tree length2 = fold (build (MINUS_EXPR, bt, ub2, lb2));
+      tree nbt;
       tree tem;
       tree comparison;
 
@@ -172,6 +200,7 @@ compare_arrays (result_type, a1, a2)
 	  tem = lb1, lb1 = lb2, lb2 = tem;
 	  tem = ub1, ub1 = ub2, ub2 = tem;
 	  tem = length1, length1 = length2, length2 = tem;
+	  tem = a1_is_null, a1_is_null = a2_is_null, a2_is_null = tem;
 	}
 
       /* If the length of this dimension in the second array is the constant
@@ -188,6 +217,8 @@ compare_arrays (result_type, a1, a2)
 	  if (contains_placeholder_p (comparison))
 	    comparison = build (WITH_RECORD_EXPR, result_type,
 				comparison, a1);
+	  if (contains_placeholder_p (length1))
+	    length1 = build (WITH_RECORD_EXPR, bt, length1, a1);
 
 	  length_zero_p = 1;
 	}
@@ -201,18 +232,20 @@ compare_arrays (result_type, a1, a2)
 	  lb1 = TYPE_MIN_VALUE (TYPE_INDEX_TYPE (TYPE_DOMAIN (t1)));
 	  ub2 = TYPE_MAX_VALUE (TYPE_INDEX_TYPE (TYPE_DOMAIN (t2)));
 	  lb2 = TYPE_MIN_VALUE (TYPE_INDEX_TYPE (TYPE_DOMAIN (t2)));
-	  bt = get_base_type (TREE_TYPE (ub1));
+	  nbt = get_base_type (TREE_TYPE (ub1));
 
 	  comparison
 	    = build_binary_op (EQ_EXPR, result_type, 
-			       build_binary_op (MINUS_EXPR, bt, ub1, lb1),
-			       build_binary_op (MINUS_EXPR, bt, ub2, lb2));
+			       build_binary_op (MINUS_EXPR, nbt, ub1, lb1),
+			       build_binary_op (MINUS_EXPR, nbt, ub2, lb2));
 
 	  /* Note that we know that UB2 and LB2 are constant and hence
 	     cannot contain a PLACEHOLDER_EXPR.  */
 
 	  if (contains_placeholder_p (comparison))
 	    comparison = build (WITH_RECORD_EXPR, result_type, comparison, a1);
+	  if (contains_placeholder_p (length1))
+	    length1 = build (WITH_RECORD_EXPR, bt, length1, a1);
 	}
 
       /* Otherwise compare the computed lengths.  */
@@ -229,6 +262,16 @@ compare_arrays (result_type, a1, a2)
 
       result = build_binary_op (TRUTH_ANDIF_EXPR, result_type,
 				result, comparison);
+      a1_is_null
+	= build_binary_op (TRUTH_ORIF_EXPR, result_type,
+			   build_binary_op (LT_EXPR, result_type, length1,
+					    convert (bt, integer_zero_node)),
+			   a1_is_null);
+      a2_is_null
+	= build_binary_op (TRUTH_ORIF_EXPR, result_type,
+			   build_binary_op (LT_EXPR, result_type, length2,
+					    convert (bt, integer_zero_node)),
+			   a2_is_null);
 
       t1 = TREE_TYPE (t1);
       t2 = TREE_TYPE (t2);
@@ -238,7 +281,7 @@ compare_arrays (result_type, a1, a2)
      data in the array.  */
   if (! length_zero_p)
     {
-      tree type = find_common_array_type (TREE_TYPE (a1), TREE_TYPE (a2));
+      tree type = find_common_type (TREE_TYPE (a1), TREE_TYPE (a2));
 
       if (type != 0)
 	a1 = convert (type, a1), a2 = convert (type, a2);
@@ -249,6 +292,19 @@ compare_arrays (result_type, a1, a2)
 			   build (EQ_EXPR, result_type, a1, a2));
     }
 
+  result = build_binary_op (TRUTH_ORIF_EXPR, result_type,
+			    build_binary_op (TRUTH_ANDIF_EXPR, result_type,
+					     a1_is_null, a2_is_null),
+			    result);
+
+  /* If either array has side-effects, they must contains SAVE_EXPRs, which
+     have to be evaluated before starting the comparison above since
+     the place it would be otherwise evaluated would be wrong.  */
+  if (TREE_SIDE_EFFECTS (a1))
+    result = build (COMPOUND_EXPR, result_type, a1, result);
+  if (TREE_SIDE_EFFECTS (a2))
+    result = build (COMPOUND_EXPR, result_type, a2, result);
+
   return result;
 }
 
@@ -371,7 +427,9 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
   tree right_type = TREE_TYPE (right_operand);
   tree left_base_type = get_base_type (left_type);
   tree right_base_type = get_base_type (right_type);
-  tree operation_type = (result_type != 0 && TYPE_EXTRA_SUBTYPE_P (result_type)
+  tree operation_type = (result_type != 0
+			 && ! AGGREGATE_TYPE_P (result_type)
+			 && TYPE_EXTRA_SUBTYPE_P (result_type)
 			 ? get_base_type (result_type) : result_type);
   tree modulus = (operation_type != 0 && TYPE_MODULAR_P (operation_type)
 		  ? TYPE_MODULUS (operation_type) : 0);
@@ -384,11 +442,14 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
       if (operation_type == 0)
 	operation_type = left_type;
 
-      /* If we are copying one array to another, find the best type to use.  */
-      if (TREE_CODE (left_type) == ARRAY_TYPE
-	  && TREE_CODE (right_type) == ARRAY_TYPE)
+      /* If we are copying one array or record to another, find the best type
+	 to use.  */
+      if ((TREE_CODE (left_type) == ARRAY_TYPE
+	   && TREE_CODE (right_type) == ARRAY_TYPE)
+	  || (TREE_CODE (left_type) == RECORD_TYPE
+	      && TREE_CODE (right_type) == RECORD_TYPE))
 	{
-	  tree best_type = find_common_array_type (left_type, right_type);
+	  tree best_type = find_common_type (left_type, right_type);
 
 	  if (best_type && left_type != best_type)
 	    left_operand = convert (best_type, left_operand);
@@ -398,7 +459,7 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 	  if (best_type)
 	    operation_type = best_type;
 	}
-      else if (TREE_TYPE (right_operand) != operation_type)
+      else
 	right_operand = convert (operation_type, right_operand);
 
       has_side_effects = 1;
@@ -409,8 +470,11 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
       if (operation_type == 0)
 	operation_type = TREE_TYPE (left_type);
 
-      if (right_type != TYPE_DOMAIN (left_type))
-	right_operand = convert (TYPE_DOMAIN (left_type), right_operand);
+      /* First convert the right operand to its base type.  This will
+	 prevent unneed signedness conversions when sizetype is wider than
+	 integer.  */
+      right_operand = convert (right_base_type, right_operand);
+      right_operand = convert (TYPE_DOMAIN (left_type), right_operand);
 
       if (! TREE_CONSTANT (right_operand)
 	  || ! TREE_CONSTANT (TYPE_MIN_VALUE (right_type)))
@@ -617,7 +681,9 @@ build_unary_op (op_code, result_type, operand)
 {
   tree type = TREE_TYPE (operand);
   tree base_type = get_base_type (type);
-  tree operation_type = (result_type != 0 && TYPE_EXTRA_SUBTYPE_P (result_type)
+  tree operation_type = (result_type != 0
+			 && ! AGGREGATE_TYPE_P (result_type)
+			 && TYPE_EXTRA_SUBTYPE_P (result_type)
 			 ? get_base_type (result_type) : result_type);
   tree result;
   int side_effects = 0;
@@ -642,14 +708,41 @@ build_unary_op (op_code, result_type, operand)
 
 	  if (type != error_mark_node)
 	    {
-	      if (TREE_CODE (type) == UNCONSTRAINED_ARRAY_TYPE)
-		type = TREE_TYPE (type);
-	      else
-		type = build_pointer_type (type);
-
+	      type = build_pointer_type (type);
 	      TREE_TYPE (result) = type;
 	    }
 	}
+      else if (TREE_CODE (operand) == ARRAY_REF)
+	{
+	  /* Compute address of array element by adding address of array
+	     to index times size of element.  */
+	  tree array = TREE_OPERAND (operand, 0);
+	  tree array_adr = build1 (ADDR_EXPR,
+				   build_pointer_type (type), array);
+	  tree low_bound = TYPE_MIN_VALUE (TYPE_DOMAIN (TREE_TYPE (array)));
+	  tree index = TREE_OPERAND (operand, 1);
+	  tree size = size_in_bytes (type);
+
+	  mark_addressable (array);
+
+	  if (TREE_CODE (low_bound) != INTEGER_CST
+	      && contains_placeholder_p (low_bound))
+	    low_bound = build (WITH_RECORD_EXPR, sizetype, low_bound, operand);
+
+	  /* Convert the integer argument to a type the same size as a
+	     pointer so the arithmetic won't overflow spuriously.  */
+	  if (TYPE_PRECISION (TREE_TYPE (index)) != POINTER_SIZE)
+	    index = convert (type_for_size (POINTER_SIZE, 0), index);
+
+	  index = fold (build (MINUS_EXPR, TREE_TYPE (index), index,
+			       convert (TREE_TYPE (index), low_bound)));
+
+	  index = fold (build (MULT_EXPR, TREE_TYPE (index),
+			       index, convert (TREE_TYPE (index), size)));
+
+	  result = fold (build (PLUS_EXPR, TYPE_POINTER_TO (type), array_adr,
+				convert (TYPE_POINTER_TO (type), index)));
+	}
       else
 	{
 	  if (type != error_mark_node)
@@ -665,12 +758,27 @@ build_unary_op (op_code, result_type, operand)
     case INDIRECT_REF:
       /* If we want to refer to an entire unconstrained array,
 	 make up an expression to do so.  This will never survive to
-	 the backend.  */
+	 the backend.  If TYPE is a thin pointer, first convert the
+	 operand to a fat pointer.  */
+      if (TREE_CODE (type) == POINTER_TYPE
+	  && TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (type))
+	  && TYPE_UNCONSTRAINED_ARRAY (TREE_TYPE (type)) != 0)
+	{
+	  operand
+	    = convert (TREE_TYPE (TYPE_UNCONSTRAINED_ARRAY (TREE_TYPE (type))),
+		       operand);
+	  type = TREE_TYPE (operand);
+	}
+
       if (TYPE_FAT_POINTER_P (type))
 	result = build1 (UNCONSTRAINED_ARRAY_REF,
 			 TYPE_UNCONSTRAINED_ARRAY (type), operand);
       else
-	result = fold (build1 (op_code, TREE_TYPE (type), operand));
+	{
+	  result = fold (build1 (op_code, TREE_TYPE (type), operand));
+	  TREE_READONLY (result) = TREE_STATIC (result)
+	    = TREE_READONLY (TREE_TYPE (type));
+	}
 
       side_effects = flag_volatile 
 	|| (! TYPE_FAT_POINTER_P (type) && TYPE_VOLATILE (TREE_TYPE (type)));
@@ -846,10 +954,32 @@ build_constructor (type, list)
 	side_effects = 1;
     }
 
+  /* If TYPE is a RECORD_TYPE and the fields are not in the
+     same order as their bit position, don't treat this as constant
+     since varasm.c can't handle it.  */
+  if (allconstant && TREE_CODE (type) == RECORD_TYPE)
+    {
+      tree last_pos = size_zero_node;
+      tree field;
+
+      for (field = TYPE_FIELDS (type); field; field = TREE_CHAIN (field))
+	{
+	  if (TREE_CODE (DECL_FIELD_BITPOS (field)) != INTEGER_CST
+	      || tree_int_cst_lt (DECL_FIELD_BITPOS (field), last_pos))
+	    {
+	      allconstant = 0;
+	      break;
+	    }
+
+	  last_pos = DECL_FIELD_BITPOS (field);
+	}
+    }
+
   result = build (CONSTRUCTOR, type, NULL_TREE, list);
   TREE_CONSTANT (result) = allconstant;
   TREE_STATIC (result) = allconstant;
   TREE_SIDE_EFFECTS (result) = side_effects;
+  TREE_READONLY (result) = TREE_READONLY (type);
 
   return result;
 }
@@ -966,7 +1096,7 @@ build_component_ref (record_variable, component, field)
 
 /* Build a GCC tree to call an allocation or deallocation function.
    If GNU_OBJ is nonzero, it is an object to deallocate.  Otherwise,
-   genrate an allocator.
+   generate an allocator.
 
    GNU_SIZE is the size of the object and ALIGN is the alignment.
    GNAT_PROC, if present is a procedure to call and GNAT_POOL is the
@@ -980,49 +1110,89 @@ build_call_alloc_dealloc (gnu_obj, gnu_size, align, gnat_proc, gnat_pool)
      Entity_Id gnat_proc;
      Entity_Id gnat_pool;
 {
-  tree gnu_align = size_int (align/ BITS_PER_UNIT);
-
-  if (TREE_CODE (gnu_size) != INTEGER_CST
-      && contains_placeholder_p (gnu_size))
-    gnu_size = build (WITH_RECORD_EXPR, sizetype, gnu_size, gnu_obj);
+  tree gnu_align = size_int (align / BITS_PER_UNIT);
 
   gnu_size = size_binop (CEIL_DIV_EXPR, gnu_size, size_int (BITS_PER_UNIT));
 
+  if (TREE_CODE (gnu_size) != INTEGER_CST && contains_placeholder_p (gnu_size))
+    gnu_size = build (WITH_RECORD_EXPR, sizetype, gnu_size,
+		      build_unary_op (INDIRECT_REF, NULL_TREE, gnu_obj));
+
   if (Present (gnat_proc))
-    {
-      /* The size is the third parameter; the alignment is the same type.  */
-      Entity_Id gnat_size_type
-	= Etype (Next_Formal (Next_Formal (First_Formal (gnat_proc))));
-      tree gnu_size_type = gnat_to_gnu_type (gnat_size_type);
-      tree gnu_proc = gnat_to_gnu_entity (gnat_proc, NULL_TREE, 0);
-      tree gnu_proc_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_proc);
-      tree gnu_pool = gnat_to_gnu_entity (gnat_pool, NULL_TREE, 0);
-      tree gnu_pool_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_pool);
-      tree gnu_args = NULL_TREE;
-      tree gnu_call;
 
-      /* The first arg is always the address of the storage pool; next
-	 comes the address of the object, for a deallocator, then the
-	 size and alignment.  */
+    /* The storage pools are obviously always tagged types, but the 
+       secondary stack uses the same mechanism and is not tagged */
 
-      gnu_args
-	= chainon (gnu_args, build_tree_list (NULL_TREE, gnu_pool_addr));
+    if (Is_Tagged_Type (Etype (gnat_pool)))
+      {
+	/* The size is the third parameter; the alignment is the same type.  */
+	Entity_Id gnat_size_type
+	  = Etype (Next_Formal (Next_Formal (First_Formal (gnat_proc))));
+	tree gnu_size_type = gnat_to_gnu_type (gnat_size_type);
+	tree gnu_proc = gnat_to_gnu_entity (gnat_proc, NULL_TREE, 0);
+	tree gnu_proc_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_proc);
+	tree gnu_pool = gnat_to_gnu_entity (gnat_pool, NULL_TREE, 0);
+	tree gnu_pool_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_pool);
+	tree gnu_args = NULL_TREE;
+	tree gnu_call;
 
-      if (gnu_obj)
-	gnu_args = chainon (gnu_args, build_tree_list (NULL_TREE, gnu_obj));
+	/* The first arg is always the address of the storage pool; next
+	   comes the address of the object, for a deallocator, then the
+	   size and alignment.  */
 
-      gnu_args = chainon (gnu_args,
-			  build_tree_list (NULL_TREE,
-					   convert (gnu_size_type, gnu_size)));
-      gnu_args = chainon (gnu_args,
-			  build_tree_list (NULL_TREE, convert (gnu_size_type,
-							       gnu_align)));
+	gnu_args
+	  = chainon (gnu_args, build_tree_list (NULL_TREE, gnu_pool_addr));
+
+	if (gnu_obj)
+	  gnu_args = chainon (gnu_args, build_tree_list (NULL_TREE, gnu_obj));
+
+	gnu_args = 
+	  chainon (gnu_args,
+		   build_tree_list (NULL_TREE,
+				    convert (gnu_size_type, gnu_size)));
+	gnu_args = 
+	  chainon (gnu_args,
+		   build_tree_list (NULL_TREE, 
+				    convert (gnu_size_type, gnu_align)));
+
+	gnu_call = build (CALL_EXPR, TREE_TYPE (TREE_TYPE (gnu_proc)),
+			  gnu_proc_addr, gnu_args, NULL_TREE);
+	TREE_SIDE_EFFECTS (gnu_call) = 1;
+	return gnu_call;
+      }
+
+    /* Secondary stack case.  */
+
+    else
+      {
+	/* The size is the second parameter */
+	Entity_Id gnat_size_type 
+	  = Etype (Next_Formal (First_Formal (gnat_proc)));
+	tree gnu_size_type = gnat_to_gnu_type (gnat_size_type);
+	tree gnu_proc = gnat_to_gnu_entity (gnat_proc, NULL_TREE, 0);
+	tree gnu_proc_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_proc);
+	tree gnu_pool = gnat_to_gnu_entity (gnat_pool, NULL_TREE, 0);
+	tree gnu_pool_addr = build_unary_op (ADDR_EXPR, NULL_TREE, gnu_pool);
+	tree gnu_args = NULL_TREE;
+	tree gnu_call;
+
+	/* The first arg is the address of the object, for a
+	   deallocator, then the size */
+
+	if (gnu_obj)
+	  gnu_args = chainon (gnu_args, build_tree_list (NULL_TREE, gnu_obj));
+
+	gnu_args = 
+	  chainon (gnu_args,
+		   build_tree_list (NULL_TREE,
+				    convert (gnu_size_type, gnu_size)));
+
+	gnu_call = build (CALL_EXPR, TREE_TYPE (TREE_TYPE (gnu_proc)),
+			  gnu_proc_addr, gnu_args, NULL_TREE);
+	TREE_SIDE_EFFECTS (gnu_call) = 1;
+	return gnu_call;
+      }
 
-      gnu_call = build (CALL_EXPR, TREE_TYPE (TREE_TYPE (gnu_proc)),
-			gnu_proc_addr, gnu_args, NULL_TREE);
-      TREE_SIDE_EFFECTS (gnu_call) = 1;
-      return gnu_call;
-    }
   else if (gnu_obj)
     return build_call_1_expr (free_decl, gnu_obj);
   else
@@ -1050,67 +1220,48 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
   tree ptr_type;
   tree result;
 
-  /* If RESULT_TYPE is a fat pointer, set SIZE to be the sum of the sizes
-     of the object and its template.  Allocate the whole thing and fill in
-     the parts that are known.  */
-  if (TYPE_FAT_POINTER_P (result_type))
+  /* If RESULT_TYPE is a fat or thin pointer, set SIZE to be the sum of the
+     sizes of the object and its template.  Allocate the whole thing and
+     fill in the parts that are known.  */
+  if (TYPE_FAT_POINTER_P (result_type)
+      || (TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (result_type))))
     {
-      tree array_type = TREE_TYPE (TREE_TYPE (TYPE_FIELDS (result_type)));
       tree template_type
-	= TREE_TYPE (TREE_TYPE (TREE_CHAIN (TYPE_FIELDS (result_type))));
-      tree template_size = TYPE_SIZE (template_type);
+	= (TYPE_FAT_POINTER_P (result_type)
+	   ? TREE_TYPE (TREE_TYPE (TREE_CHAIN (TYPE_FIELDS (result_type))))
+	   : TREE_TYPE (TYPE_FIELDS (TREE_TYPE (result_type))));
+      tree storage_type = build_unc_object_type (template_type, type, "ALLOC");
+      tree storage_ptr_type = build_pointer_type (storage_type);
       tree storage;
-      tree new_template_ptr, new_array_ptr;
+      tree template_cons = NULL_TREE;
+
+      size = TYPE_SIZE (storage_type);
 
       if (TREE_CODE (size) != INTEGER_CST
 	  && contains_placeholder_p (size))
 	size = build (WITH_RECORD_EXPR, sizetype, size, init);
 
-      size = size_binop (PLUS_EXPR, size, template_size);
-
       storage = build_call_alloc_dealloc (NULL_TREE, size,
-					  MAX (TYPE_ALIGN (array_type),
-					       TYPE_ALIGN (template_type)),
+					  TYPE_ALIGN (storage_type),
 					  gnat_proc, gnat_pool);
-      storage = save_expr (storage);
-
-      /* The template is at the start of the allocated storage followed
-	 by the array itself.  */
-      new_template_ptr = convert (build_pointer_type (template_type), storage);
-
-      new_array_ptr
-	= convert (build_pointer_type (array_type),
-		   build (PLUS_EXPR, TREE_TYPE (storage), storage,
-			  convert (TREE_TYPE (storage),
-				   size_in_bytes (template_type))));
-
-      /* Copy the template and array (if initializer).  */
-      new_template_ptr
-	= build (COMPOUND_EXPR, build_pointer_type (template_type),
-		 build_binary_op (MODIFY_EXPR, template_type,
-				  build_unary_op (INDIRECT_REF, NULL_TREE,
-						  new_template_ptr),
-				  build_template (template_type, type, init)),
-		 new_template_ptr);
+      storage = convert (storage_ptr_type, storage);
 
       if (init != 0)
-	new_array_ptr
-	  = build (COMPOUND_EXPR, build_pointer_type (array_type),
-		   build_binary_op (MODIFY_EXPR, array_type,
-				    build_unary_op (INDIRECT_REF, NULL_TREE,
-						    new_array_ptr),
-				    build_unary_op (INDIRECT_REF, NULL_TREE,
-						    build_unary_op (ADDR_EXPR,
-								    NULL_TREE,
-								    init))),
-		   new_array_ptr);
+	template_cons = tree_cons (TREE_CHAIN (TYPE_FIELDS (storage_type)),
+				   init, NULL_TREE);
+      template_cons = tree_cons (TYPE_FIELDS (storage_type),
+				 build_template (template_type, type, init),
+				 template_cons);
 
-      return
-	build_constructor
-	  (result_type,
-	   tree_cons (TYPE_FIELDS (result_type), new_array_ptr,
-		      tree_cons (TREE_CHAIN (TYPE_FIELDS (result_type)),
-				 new_template_ptr, NULL_TREE)));
+      return convert
+	(result_type,
+	 build_unary_op
+	 (ADDR_EXPR, NULL_TREE,
+	  build_binary_op
+	  (MODIFY_EXPR, storage_type,
+	   build_unary_op (INDIRECT_REF, NULL_TREE,
+			   convert (storage_ptr_type, storage)),
+	   build_constructor (storage_type, template_cons))));
     }
 
   /* If we have an initializing expression, see if its size is simpler
@@ -1136,7 +1287,8 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
   /* If we are at top-level and SIZE is a constant, we can actually
      allocate an object of TYPE and point to it.
      ??? At some point, we should make an attempt to do this above.  */
-  if (global_bindings_p () && TREE_CODE (size) == INTEGER_CST)
+  if (global_bindings_p () && TREE_CODE (size) == INTEGER_CST
+      && ! TREE_OVERFLOW (size) && 0)
     {
       char name[20];
 
@@ -1147,10 +1299,17 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
       init = 0;
     }
   else
-    result = convert (result_type,
-		      build_call_alloc_dealloc (NULL_TREE, size,
-						TYPE_ALIGN (type),
-						gnat_proc, gnat_pool));
+    {
+      /* If the size overflows, pass -1 so the allocator will raise
+	 storage error.  */
+      if (TREE_CODE (size) == INTEGER_CST && TREE_OVERFLOW (size))
+	size = convert (sizetype, build_int_2 (-BITS_PER_UNIT, ~0));
+
+      result = convert (result_type,
+			build_call_alloc_dealloc (NULL_TREE, size,
+						  TYPE_ALIGN (type),
+						  gnat_proc, gnat_pool));
+    }
 
   /* If we have an initial value, put the new address into a SAVE_EXPR, assign
      the value, and return the address.  Do this with a COMPOUND_EXPR.  */
@@ -1166,18 +1325,15 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
 		      result);
     }
 
-  /* If the result is not of the proper type, convert it.  */
-  if (TREE_TYPE (result) != result_type)
-    result = convert (result_type, result);
-
-  return result;
+  return convert (result_type, result);
 }
 
 /* Indicate that we need to make the address of EXPR_NODE and it therefore
    should not be allocated in a register. Return 1 if successful.  */
 
 int
-mark_addressable (tree expr_node)
+mark_addressable (expr_node)
+     tree expr_node;
 {
   while (1)
     switch (TREE_CODE (expr_node))
@@ -1195,7 +1351,6 @@ mark_addressable (tree expr_node)
 	return 1;
 
       case VAR_DECL:
-      case CONST_DECL:
       case PARM_DECL:
       case RESULT_DECL:
 	put_var_into_stack (expr_node);
