@@ -1,14 +1,14 @@
 /****************************************************************************/
 /*                                                                          */
-/*                         GNAT COMPILER COMPONENTS                         */
+/*                           GNAT COMPILER TOOLS                            */
 /*                                                                          */
 /*                               G N A T B L                                */
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                             $Revision: 1.44 $                            */
+/*                             $Revision: 1.50 $                            */
 /*                                                                          */
-/*           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          */
+/*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -18,7 +18,11 @@
 /* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
 /* for  more details.  You should have  received  a copy of the GNU General */
 /* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
@@ -37,6 +41,9 @@
 char *binder_path = 0;
 char *linker_path = 0;
 char *exec_file_name = 0;
+#define BIND_ARG_MAX 512
+char *bind_args[BIND_ARG_MAX];
+int  bind_arg_index = -1;
 #ifdef MSDOS
 char *coff2exe_path = 0;
 char *coff2exe_args[] = {(char *) 0, (char *) 0};
@@ -49,11 +56,6 @@ int  g_present    = 0;
 int  link_arg_max = -1;
 char **link_args = (char **) 0;
 int  link_arg_index = -1;
-int  numerics_present = 0;  /* indicates presence of ada numerics packages */
-int  tasking_present = 0;   /* indicates needs for gthreads library */
-#ifdef WINNT
-int  windows_present = 0;  /* indicates a windows application */
-#endif
 
 char *gcc_B_arg = 0;
 #ifndef DIR_SEPARATOR
@@ -95,6 +97,17 @@ process_args (int *p_argc, char *argv [])
 
   for (i = 1; i < *p_argc; i++) {
 
+    /* -I is passed on to gnatbind */
+    if (! strncmp( argv[i], "-I", 2))
+    {
+      bind_arg_index += 1;
+      if (bind_arg_index >= BIND_ARG_MAX)
+      {
+         fprintf (stderr, "Too many arguments to gnatbind\n");
+         exit (-1);
+      }
+      bind_args[bind_arg_index] = argv[i];
+    }
     /* -B is passed on to gcc */
     if (! strncmp (argv [i], "-B", 2)) {
        gcc_B_arg = argv[i];
@@ -171,7 +184,7 @@ process_args (int *p_argc, char *argv [])
 
 main (int argc, char *argv [])
 {
-  int i;
+  int i,j;
   int done_an_ali = 0;
   int file_name_index;
   int retcode;
@@ -248,7 +261,9 @@ main (int argc, char *argv [])
           spawn_index = 0;
 	  spawn_args [spawn_index++] = binder_path;
 	  spawn_args [spawn_index++] = argv[i];
-	  spawn_args [spawn_index] = 0;
+	  for (j = 0 ; j <= bind_arg_index ; j++ )
+	    spawn_args [spawn_index++] = bind_args [j];
+ 	  spawn_args [spawn_index] = 0;
 
           if (verbose) {
             int i;
@@ -354,20 +369,6 @@ main (int argc, char *argv [])
 #endif
   }
 
-#if !(defined (__EMX__))
-#if !(defined (MSDOS) || defined (WINNT))
-  addarg ("-L" ADA_RTL_OBJ_DIR);
-  if (tasking_present) 
-    addarg ("-lgthreads");
-#endif
-#ifndef WINNT
-  if (numerics_present) addarg ("-lm");
-#endif
-#endif
-#ifdef WINNT
-  if (windows_present)
-      addarg ("-mwindows");
-#endif
   addarg ("-lgnat");
   addarg (NULL);
 
@@ -466,35 +467,6 @@ proc_bind_file (char *bind_file_name)
     addarg (xstrdup (buf));
 #endif
 
-    /* If the object file "a-numaux" is present set a flag and later
-     * when the final arguments of the link are appended, the flag will be
-     * queried and "-lm" will be added to the link since this is needed by
-     * the numerics packages.
-     */
-    ptr = rindex (buf, '/');
-    if (ptr != NULL) {
-      ptr++;  /* skip over the separator */
-    }
-    else {
-      ptr = buf;
-    }
-    if ('/' != DIR_SEPARATOR) {  /* DJGPP could have either or both */
-      tptr = rindex (ptr, DIR_SEPARATOR);
-      if (tptr != NULL) {
-        ptr = ++tptr;  /* skip over the separator */
-      }
-    }
-    if (! strcmp (ptr, "a-numaux.o"))
-      numerics_present++;
-
-    if (! strcmp (ptr, "s-taskin.o"))
-      tasking_present++;
-
-#ifdef WINNT
-    if (! strcmp (ptr, "windows.o"))
-      windows_present++;
-#endif
-
   }
 #ifdef WINNT
   fclose (argfile);
