@@ -1,14 +1,14 @@
 /****************************************************************************/
 /*                                                                          */
-/*                         GNAT COMPILER COMPONENTS                         */
+/*                           GNAT COMPILER TOOLS                            */
 /*                                                                          */
 /*                              G N A T C H P                               */
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                             $Revision: 1.18 $                            */
+/*                             $Revision: 1.22 $                            */
 /*                                                                          */
-/*           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          */
+/*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -18,7 +18,11 @@
 /* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
 /* for  more details.  You should have  received  a copy of the GNU General */
 /* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
@@ -106,7 +110,6 @@ main (int argc, char *argv [])
   int  source_ref_option = 0;
   int  script_option = 0;
   int  errors = 0;
-  char buf [10000];
   char *ptr;
   char name [10000];
   char *names [MAX_UNITS];
@@ -122,14 +125,14 @@ main (int argc, char *argv [])
   extern char *optarg;
   extern int optind;
 
-  /* Scan the arguments list. The arguments for gnatsplit would be 
-   * gnatsplit [-ksw] filename [directory]
+  /* Scan the arguments list. The arguments for gnatchop would be 
+   * gnatchop [-ksw] filename [directory]
    */
 
   while ((c = getopt(argc, argv, "krsw")) != -1)
      switch (c) {
         case 'k':
-            /* passed from k8 option to gnatsplit, ignore it */
+            /* passed from k8 option to gnatchop, ignore it */
             break;
         case 's':
             script_option++;
@@ -142,7 +145,7 @@ main (int argc, char *argv [])
             break;
         case '?':
             fprintf
-              (stderr, "usage: gnatsplit [-krsw] filename [directory]\n");
+              (stderr, "usage: gnatchop [-krsw] filename [directory]\n");
             exit (1);
      }
 
@@ -165,51 +168,61 @@ main (int argc, char *argv [])
   /* It is assumed that gcc is called by "gcc -gnatu -gnats -c filename" on 
    * a source file and the subsequent output is redirected as the standard
    * input to this program.  The strings are of the following forms:
+   * "Config pragmas line 1, file offset 0, file name gnat.adc"
    * "Unit XXXXX (spec) line ddd, file offset ddd. file name XXXX.ads" or
    * "Unit XXXXX (body) line ddd, file offset ddd. file name XXXX.adb"
    * ...
    */
 
   printf("splitting %s into: \n", source_name);
-  read (0, buf, 10000);
-  ptr = buf;
-
-  for (unit=1;;unit++) {
-    /* Scan for the string ") line", skip over it, and read the line number.
-     * if the string does not appear, it indicates that the entire string
-     * has been fully processed.
+  {
+    /* Declare a buffer big enough to hold the offset information and then
+     * read it all in with a single read operation.
      */
-     ptr = strstr (ptr, ") line");
-     if (ptr == (char *) 0) break;
-     ptr = ptr + 6;
-     sscanf(ptr, "%d", &n);
-     linenums [unit] = n;
+    int buffer_size = file_length (0);
+    char buffer [buffer_size+1];
 
-    /* Scan for the word "offset", skip over it and read the actual offset
-     * must be found, since we just found a ) line before it
-     */
-    ptr = strstr (ptr, "offset"); 
-    ptr = ptr + 6;
-    sscanf(ptr, "%d", &n);
-    offsets [unit] = n;
+    read (0, buffer, buffer_size);
+    buffer [buffer_size] = 0;
+    ptr = buffer;
+    for (unit=1;;unit++) {
+      /* Scan for the string " line", skip over it, and read the line number.
+       * if the string does not appear, it indicates that the entire string
+       * has been fully processed.
+       */
+       ptr = strstr (ptr, " line");
+       if (ptr == (char *) 0) break;
+       ptr = ptr + 6;
+       sscanf(ptr, "%d", &n);
+       linenums [unit] = n;
 
-    /* Scan for the words "file name", skip over them and read in the actual
-     * file name to be used when writing out this compilation unit.  If a 
-     * directory name was given as an argument, prepend it to the file name.
-     */
-    ptr = strstr (ptr, "file name"); 
-    ptr = ptr + 9;
-    sscanf(ptr, "%s", &name);
-    names [unit] = malloc (strlen (name) + directory_len + 1);
-    names [unit] [0] = '\0';
-    if (directory) {
-      strcpy (names [unit], directory);
-      i = strlen (names [unit]);
-      names [unit] [i] = DIR_SEPARATOR;
-      names [unit] [i+1] = '\0';
+      /* Scan for the word "offset", skip over it and read the actual offset
+       * must be found, since we just found a ) line before it
+       */
+      ptr = strstr (ptr, "offset"); 
+      ptr = ptr + 6;
+      sscanf(ptr, "%d", &n);
+      offsets [unit] = n;
+
+      /* Scan for the words "file name", skip over them and read in the actual
+       * file name to be used when writing out this compilation unit.  If a 
+       * directory name was given as an argument, prepend it to the file name.
+       */
+      ptr = strstr (ptr, "file name"); 
+      ptr = ptr + 9;
+      sscanf(ptr, "%s", &name);
+      names [unit] = malloc (strlen (name) + directory_len + 1);
+      names [unit] [0] = '\0';
+      if (directory) {
+        strcpy (names [unit], directory);
+        i = strlen (names [unit]);
+        names [unit] [i] = DIR_SEPARATOR;
+        names [unit] [i+1] = '\0';
+      }
+      strcat (names [unit], name);
+      printf("   %s \n", names [unit]);
+      ptr = strstr (ptr, "\n"); 
     }
-    strcat (names [unit], name);
-    printf("   %s \n", names [unit]);
   }
 
   printf("\n"); 
