@@ -22,7 +22,7 @@ Report problems and direct all questions to:
 
   pthreads-bugs@ada.cs.fsu.edu
 
-  @(#)unistd.h	2.1 3/16/94
+  @(#)unistd.h	2.5 4/12/95
 
 */
 
@@ -33,12 +33,17 @@ Report problems and direct all questions to:
 
 #include <unistd.h>
 
+/*
+ * We need to figure out (from header files) what system we are running on.
+ */
 #ifdef _UNISTD_H
-#ifndef SOLARIS
-#define SOLARIS
-#endif
-#elif defined(SOLARIS)
-#undef SOLARIS
+#ifndef SOLARIS_NP
+#define SOLARIS_NP
+#define SVR4_NP
+#endif _UNISTD_H
+#elif defined(SOLARIS_NP) || defined(SVR4_NP)
+#undef SOLARIS_NP
+#undef SVR4_NP
 #endif _UNISTD_H
 
 #endif !LOCORE
@@ -47,7 +52,11 @@ Report problems and direct all questions to:
 #define	_POSIX_THREADS
 #define	_POSIX_THREAD_PRIORITY_SCHEDULING
 #define _POSIX_THREAD_ATTR_STACKSIZE
+#ifdef SRP_NP
 #define _POSIX_THREADS_PRIO_PROTECT
+#else !SRP_NP
+#undef _POSIX_THREADS_PRIO_PROTECT
+#endif !SRP_NP
 #undef _POSIX_THREADS_PROCESS_SHARED
 #undef _POSIX_THREADS_PRIO_INHERIT
 #undef _POSIX_REENTRANT_FUNCTIONS
